@@ -1,7 +1,7 @@
 /* Swfdec
  * Copyright (C) 2003-2006 David Schleef <ds@schleef.org>
  *		 2005-2006 Eric Anholt <eric@anholt.net>
- *		      2006 Benjamin Otte <otte@gnome.org>
+ *		 2006-2007 Benjamin Otte <otte@gnome.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -22,10 +22,17 @@
 #ifndef __SWFDEC_H__
 #define __SWFDEC_H__
 
+#include <libswfdec/swfdec_as_context.h>
+#include <libswfdec/swfdec_as_frame.h>
+#include <libswfdec/swfdec_as_object.h>
+#include <libswfdec/swfdec_as_types.h>
+
 #include <libswfdec/swfdec_audio.h>
 #include <libswfdec/swfdec_buffer.h>
 #include <libswfdec/swfdec_enums.h>
+#include <libswfdec/swfdec_file_loader.h>
 #include <libswfdec/swfdec_loader.h>
 #include <libswfdec/swfdec_player.h>
+#include <libswfdec/swfdec_url.h>
 
 #endif
