@@ -307,11 +307,9 @@ swfdec_bits_get_float (SwfdecBits * b)
 
   SWFDEC_BYTES_CHECK (b, 4);
 
-  conv.i = *((gint32 *) b->ptr);
+  conv.i = (b->ptr[3] << 24) | (b->ptr[2] << 16) | (b->ptr[1] << 8) | b->ptr[0];
   b->ptr += 4;
 
-  conv.i = GINT32_FROM_LE (conv.i);
-
   return conv.f;
 }
 
@@ -323,40 +321,30 @@ swfdec_bits_get_float (SwfdecBits * b)
  * use this command line:
  * python -c "import struct; print struct.unpack('8c', struct.pack('d', 7.949928895127363e-275))"
  */
-static double 
-swfdec_bits_double_to_host (double in)
+double
+swfdec_bits_get_double (SwfdecBits * b)
 {
   union {
     guint32 i[2];
     double d;
   } conv;
 
-  conv.d = in;
-#if G_BYTE_ORDER == G_LITTLE_ENDIAN
-  {
-    int tmp = conv.i[0];
-    conv.i[0] = conv.i[1];
-    conv.i[1] = tmp;
-  }
-#else
-  conv.i[0] = GUINT32_FROM_LE (conv.i[0]);
-  conv.i[1] = GUINT32_FROM_LE (conv.i[1]);
-#endif
-  return conv.d;
-}
-
-double
-swfdec_bits_get_double (SwfdecBits * b)
-{
-  double d;
-
   SWFDEC_BYTES_CHECK (b, 8);
 
-  d = *((double *) b->ptr);
+#if G_BYTE_ORDER == G_LITTLE_ENDIAN
+  conv.i[1] = (b->ptr[3] << 24) | (b->ptr[2] << 16) | (b->ptr[1] << 8) | b->ptr[0];
+  conv.i[0] = (b->ptr[7] << 24) | (b->ptr[6] << 16) | (b->ptr[5] << 8) | b->ptr[4];
+#else
+  conv.i[0] = (b->ptr[3] << 24) | (b->ptr[2] << 16) | (b->ptr[1] << 8) | b->ptr[0];
+  conv.i[1] = (b->ptr[7] << 24) | (b->ptr[6] << 16) | (b->ptr[5] << 8) | b->ptr[4];
+#if 0
+  conv.i[0] = (b->ptr[0] << 24) | (b->ptr[1] << 16) | (b->ptr[2] << 8) | b->ptr[3];
+  conv.i[1] = (b->ptr[4] << 24) | (b->ptr[5] << 16) | (b->ptr[6] << 8) | b->ptr[7];
+#endif
+#endif
   b->ptr += 8;
-  d = swfdec_bits_double_to_host (d);
 
-  return d;
+  return conv.d;
 }
 
 double
@@ -470,22 +458,14 @@ swfdec_bits_get_matrix (SwfdecBits * bits, cairo_matrix_t *matrix,
   swfdec_bits_syncbits (bits);
 }
 
-char *
-swfdec_bits_get_string (SwfdecBits * bits)
-{
-  const char *s = swfdec_bits_skip_string (bits);
-
-  return g_strdup (s);
-}
-
-const char *
+static const char *
 swfdec_bits_skip_string (SwfdecBits *bits)
 {
   char *s;
   const char *end;
   guint len;
 
-  swfdec_bits_syncbits (bits);
+  SWFDEC_BYTES_CHECK (bits, 1);
   end = memchr (bits->ptr, 0, bits->end - bits->ptr);
   if (end == NULL) {
     SWFDEC_ERROR ("could not parse string");
@@ -495,14 +475,53 @@ swfdec_bits_skip_string (SwfdecBits *bits)
   len = end - (const char *) bits->ptr;
   s = (char *) bits->ptr;
   bits->ptr += len + 1;
-  if (!g_utf8_validate (s, -1, NULL)) {
-    SWFDEC_ERROR ("parsed string is not valid utf-8");
-    s = NULL;
-  }
 
   return s;
 }
 
+/**
+ * swfdec_bits_get_string_with_version:
+ * @bits: a #SwfdecBits
+ * @version: Flash player version
+ *
+ * Prior to Flash 6, strings used to be encoded as LATIN1. Since Flash 6, 
+ * strings are encoded as UTF-8. This version does the check automatically
+ * and converts strings to UTF-8.
+ *
+ * Returns: a UTF-8 encoded string or %NULL on error
+ **/
+char *
+swfdec_bits_get_string_with_version (SwfdecBits *bits, guint version)
+{
+  const char *s;
+  
+  g_return_val_if_fail (bits != NULL, NULL);
+
+  s = swfdec_bits_skip_string (bits);
+  if (s == NULL)
+    return NULL;
+
+  if (version < 6) {
+    char *ret = g_convert (s, -1, "UTF-8", "LATIN1", NULL , NULL, NULL);
+    if (ret == NULL)
+      g_warning ("Could not convert string from LATIN1 to UTF-8");
+    return ret;
+  } else {
+    if (!g_utf8_validate (s, -1, NULL)) {
+      SWFDEC_ERROR ("parsed string is not valid utf-8");
+      return NULL;
+    }
+    return g_strdup (s);
+  }
+}
+
+/* FIXME: deprecated - someone remove this */
+char *
+swfdec_bits_get_string (SwfdecBits * bits)
+{
+  return swfdec_bits_get_string_with_version (bits, 6);
+}
+
 /**
  * swfdec_bits_skip_bytes:
  * @bits: a #SwfdecBits
