@@ -267,15 +267,15 @@ swfdec_flv_decoder_parse_video_tag (SwfdecFlvDecoder *flv, SwfdecBits *bits, gui
     g_array_insert_val (flv->video, idx, tag);
   }
   if (dec->width == 0 && dec->height == 0) {
-    SwfdecFlvVideoTag *tag = &g_array_index (flv->video, SwfdecFlvVideoTag, 0);
+    SwfdecFlvVideoTag *t = &g_array_index (flv->video, SwfdecFlvVideoTag, 0);
     SwfdecVideoDecoder *decoder;
     cairo_surface_t *surface;
 
     /* nice hack... */
-    decoder = swfdec_video_decoder_new (tag->format);
+    decoder = swfdec_video_decoder_new (t->format);
     if (decoder == NULL)
       return SWFDEC_STATUS_OK;
-    surface = swfdec_video_decoder_decode (decoder, tag->buffer);
+    surface = swfdec_video_decoder_decode (decoder, t->buffer);
     if (surface == NULL)
       return SWFDEC_STATUS_OK;
     dec->width = cairo_image_surface_get_width (surface);
@@ -380,8 +380,9 @@ swfdec_flv_decoder_parse_tag (SwfdecFlvDecoder *flv)
   swfdec_bits_init (&bits, buffer);
   type = swfdec_bits_get_u8 (&bits);
   /* I think I'm paranoid and complicated. I think I'm paranoid, manipulated */
-  if (size != swfdec_bits_get_bu24 (&bits))
+  if (size != swfdec_bits_get_bu24 (&bits)) {
     g_assert_not_reached ();
+  }
   timestamp = swfdec_bits_get_bu24 (&bits);
   swfdec_bits_get_bu32 (&bits);
   SWFDEC_LOG ("new tag");
@@ -605,10 +606,10 @@ swfdec_flv_decoder_get_data (SwfdecFlvDecoder *flv, guint timestamp, guint *real
 
 #include "swfdec_loadertarget.h"
 #include "swfdec_net_stream.h"
-#include "swfdec_root_movie.h"
 #include "swfdec_sprite.h"
 #include "swfdec_video_movie.h"
 
+#if 0
 static void
 notify_initialized (SwfdecPlayer *player, GParamSpec *pspec, SwfdecVideoMovie *movie)
 {
@@ -618,6 +619,7 @@ notify_initialized (SwfdecPlayer *player, GParamSpec *pspec, SwfdecVideoMovie *m
   swfdec_movie_queue_update (SWFDEC_MOVIE (movie), SWFDEC_MOVIE_INVALID_MATRIX);
   swfdec_movie_invalidate (SWFDEC_MOVIE (movie));
 }
+#endif
 
 gboolean
 swfdec_flv_decoder_is_eof (SwfdecFlvDecoder *flv)
@@ -638,6 +640,9 @@ swfdec_flv_decoder_eof (SwfdecFlvDecoder *flv)
 SwfdecMovie *
 swfdec_flv_decoder_add_movie (SwfdecFlvDecoder *flv, SwfdecMovie *parent)
 {
+  //g_assert_not_reached ();
+  return NULL;
+#if 0
   SwfdecContent *content = swfdec_content_new (0);
   SwfdecMovie *movie;
   SwfdecVideo *video;
@@ -649,8 +654,8 @@ swfdec_flv_decoder_add_movie (SwfdecFlvDecoder *flv, SwfdecMovie *parent)
   video->width = G_MAXUINT;
   video->height = G_MAXUINT;
   content->graphic = SWFDEC_GRAPHIC (video);
+  content->free = TRUE;
   movie = swfdec_movie_new (parent, content);
-  g_object_weak_ref (G_OBJECT (movie), (GWeakNotify) swfdec_content_free, content);
   g_object_weak_ref (G_OBJECT (movie), (GWeakNotify) g_object_unref, video);
   g_signal_connect (SWFDEC_ROOT_MOVIE (parent)->player, "notify::initialized",
       G_CALLBACK (notify_initialized), movie);
@@ -665,5 +670,6 @@ swfdec_flv_decoder_add_movie (SwfdecFlvDecoder *flv, SwfdecMovie *parent)
   g_object_unref (stream);
 
   return movie;
+#endif
 }
 
