@@ -69,6 +69,18 @@
  * inside a #SwfdecLoader.
  */
 
+/**
+ * SwfdecLoaderRequest:
+ * @SWFDEC_LOADER_REQUEST_DEFAULT: Use the default method (this most likely is 
+ *                                 equal to HTTPget)
+ * @SWFDEC_LOADER_REQUEST_GET: Use HTTP get
+ * @SWFDEC_LOADER_REQUEST_POST: Use HTTP post
+ *
+ * Describes the moethod to use for requesting a given URL. These methods map
+ * naturally to HTTP methods, since HTTP is the common method for requesting 
+ * Flash content.
+ */
+
 /*** SwfdecLoader ***/
 
 enum {
@@ -77,7 +89,8 @@ enum {
   PROP_EOF,
   PROP_DATA_TYPE,
   PROP_SIZE,
-  PROP_LOADED
+  PROP_LOADED,
+  PROP_URL
 };
 
 G_DEFINE_ABSTRACT_TYPE (SwfdecLoader, swfdec_loader, G_TYPE_OBJECT)
@@ -93,7 +106,7 @@ swfdec_loader_get_property (GObject *object, guint param_id, GValue *value,
       g_value_set_string (value, loader->error);
       break;
     case PROP_EOF:
-      g_value_set_boolean (value, loader->eof);
+      g_value_set_boolean (value, loader->state == SWFDEC_LOADER_STATE_EOF);
       break;
     case PROP_DATA_TYPE:
       g_value_set_enum (value, loader->data_type);
@@ -104,6 +117,9 @@ swfdec_loader_get_property (GObject *object, guint param_id, GValue *value,
     case PROP_LOADED:
       g_value_set_ulong (value, swfdec_loader_get_loaded (loader));
       break;
+    case PROP_URL:
+      g_value_set_boxed (value, loader->url);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
       break;
@@ -120,14 +136,17 @@ swfdec_loader_set_property (GObject *object, guint param_id, const GValue *value
     case PROP_ERROR:
       swfdec_loader_error (loader, g_value_get_string (value));
       break;
-    case PROP_EOF:
-      if (g_value_get_boolean (value) && !loader->eof)
-	swfdec_loader_eof (loader);
-      break;
     case PROP_SIZE:
       if (loader->size == 0 && g_value_get_ulong (value) > 0)
 	swfdec_loader_set_size (loader, g_value_get_ulong (value));
       break;
+    case PROP_URL:
+      loader->url = g_value_dup_boxed (value);
+      if (loader->url == NULL) {
+	g_warning ("must set a valid URL");
+	loader->url = swfdec_url_new ("");
+      }
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
       break;
@@ -139,8 +158,10 @@ swfdec_loader_dispose (GObject *object)
 {
   SwfdecLoader *loader = SWFDEC_LOADER (object);
 
+  /* targets are supposed to keep a reference around */
+  g_assert (loader->target == NULL);
   swfdec_buffer_queue_unref (loader->queue);
-  g_free (loader->url);
+  swfdec_url_free (loader->url);
   g_free (loader->error);
 
   G_OBJECT_CLASS (swfdec_loader_parent_class)->dispose (object);
@@ -167,9 +188,12 @@ swfdec_loader_class_init (SwfdecLoaderClass *klass)
   g_object_class_install_property (object_class, PROP_SIZE,
       g_param_spec_ulong ("size", "size", "amount of bytes in loader",
 	  0, G_MAXULONG, 0, G_PARAM_READWRITE));
-  g_object_class_install_property (object_class, PROP_SIZE,
+  g_object_class_install_property (object_class, PROP_LOADED,
       g_param_spec_ulong ("loaded", "loaded", "bytes already loaded",
 	  0, G_MAXULONG, 0, G_PARAM_READWRITE));
+  g_object_class_install_property (object_class, PROP_URL,
+      g_param_spec_boxed ("url", "url", "URL for this file",
+	  SWFDEC_TYPE_URL, G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 }
 
 static void
@@ -179,83 +203,58 @@ swfdec_loader_init (SwfdecLoader *loader)
   loader->data_type = SWFDEC_LOADER_DATA_UNKNOWN;
 }
 
-/*** SwfdecFileLoader ***/
-
-G_DEFINE_TYPE (SwfdecFileLoader, swfdec_file_loader, SWFDEC_TYPE_LOADER)
-
-static void
-swfdec_file_loader_dispose (GObject *object)
-{
-  SwfdecFileLoader *file_loader = SWFDEC_FILE_LOADER (object);
-
-  g_free (file_loader->dir);
-
-  G_OBJECT_CLASS (swfdec_file_loader_parent_class)->dispose (object);
-}
-
-static SwfdecLoader *
-swfdec_file_loader_load (SwfdecLoader *loader, const char *url)
-{
-  SwfdecBuffer *buffer;
-  char *real_path;
-  SwfdecLoader *ret;
-  GError *error = NULL;
-
-  if (g_path_is_absolute (url)) {
-    SWFDEC_ERROR ("\"%s\" is an absolute path - using relative instead", url);
-    while (*url == G_DIR_SEPARATOR)
-      url++;
-  }
-
-  /* FIXME: need to rework seperators on windows? */
-  real_path = g_build_filename (SWFDEC_FILE_LOADER (loader)->dir, url, NULL);
-  buffer = swfdec_buffer_new_from_file (real_path, &error);
-  ret = g_object_new (SWFDEC_TYPE_FILE_LOADER, NULL);
-  ret->url = real_path;
-  SWFDEC_FILE_LOADER (ret)->dir = g_strdup (SWFDEC_FILE_LOADER (loader)->dir);
-  if (buffer == NULL) {
-    swfdec_loader_error (ret, error->message);
-    g_error_free (error);
-  } else {
-    swfdec_loader_set_size (ret, buffer->length);
-    swfdec_loader_push (ret, buffer);
-    swfdec_loader_eof (ret);
-  }
-
-  return ret;
-}
-
-static void
-swfdec_file_loader_class_init (SwfdecFileLoaderClass *klass)
-{
-  GObjectClass *object_class = G_OBJECT_CLASS (klass);
-  SwfdecLoaderClass *loader_class = SWFDEC_LOADER_CLASS (klass);
-
-  object_class->dispose = swfdec_file_loader_dispose;
-
-  loader_class->load = swfdec_file_loader_load;
-}
-
-static void
-swfdec_file_loader_init (SwfdecFileLoader *loader)
-{
-}
-
 /*** INTERNAL API ***/
 
+static void
+swfdec_loader_perform_open (gpointer loaderp, gpointer unused)
+{
+  SwfdecLoader *loader = loaderp;
+
+  swfdec_loader_target_open (loader->target, loader);
+}
+
+static void
+swfdec_loader_perform_eof (gpointer loaderp, gpointer unused)
+{
+  SwfdecLoader *loader = loaderp;
+
+  swfdec_loader_target_eof (loader->target, loader);
+}
+
+static void
+swfdec_loader_perform_error (gpointer loaderp, gpointer unused)
+{
+  SwfdecLoader *loader = loaderp;
+
+  swfdec_loader_target_error (loader->target, loader);
+}
+
+static void
+swfdec_loader_perform_push (gpointer loaderp, gpointer unused)
+{
+  SwfdecLoader *loader = loaderp;
+
+  swfdec_loader_target_parse (loader->target, loader);
+}
+
 SwfdecLoader *
-swfdec_loader_load (SwfdecLoader *loader, const char *url)
+swfdec_loader_load (SwfdecLoader *loader, const char *url_string,
+    SwfdecLoaderRequest request, const char *data, gsize data_len)
 {
   SwfdecLoader *ret;
   SwfdecLoaderClass *klass;
+  SwfdecURL *url;
 
   g_return_val_if_fail (SWFDEC_IS_LOADER (loader), NULL);
-  g_return_val_if_fail (url != NULL, NULL);
+  g_return_val_if_fail (url_string != NULL, NULL);
+  g_return_val_if_fail (data != NULL || data_len == 0, NULL);
 
   klass = SWFDEC_LOADER_GET_CLASS (loader);
   g_return_val_if_fail (klass->load != NULL, NULL);
-  ret = klass->load (loader, url);
-  g_assert (ret != NULL);
+  url = swfdec_url_new_relative (loader->url, url_string);
+  ret = g_object_new (G_OBJECT_CLASS_TYPE (klass), "url", url, NULL);
+  swfdec_url_free (url);
+  klass->load (ret, loader, request, data, data_len);
   return ret;
 }
 
@@ -265,72 +264,48 @@ swfdec_loader_set_target (SwfdecLoader *loader, SwfdecLoaderTarget *target)
   g_return_if_fail (SWFDEC_IS_LOADER (loader));
   g_return_if_fail (target == NULL || SWFDEC_IS_LOADER_TARGET (target));
 
+  if (loader->target) {
+    swfdec_player_remove_all_external_actions (loader->player, loader);
+  }
   loader->target = target;
-}
-
-static void
-swfdec_loader_do_parse (gpointer empty, gpointer loaderp)
-{
-  SwfdecLoader *loader = SWFDEC_LOADER (loaderp);
-
-  swfdec_loader_target_parse (loader->target, loader);
-}
-
-void
-swfdec_loader_queue_parse (SwfdecLoader *loader)
-{
-  SwfdecPlayer *player;
-
-  g_return_if_fail (SWFDEC_IS_LOADER (loader));
-  g_return_if_fail (loader->target != NULL);
-
-  player = swfdec_loader_target_get_player (loader->target);
-  /* HACK: using player as action object makes them get auto-removed */
-  swfdec_player_add_action (player, player, swfdec_loader_do_parse, loader);
+  if (target) {
+    loader->player = swfdec_loader_target_get_player (target);
+    switch (loader->state) {
+      case SWFDEC_LOADER_STATE_NEW:
+	break;
+      case SWFDEC_LOADER_STATE_OPEN:
+	swfdec_player_add_external_action (loader->player, loader,
+	    swfdec_loader_perform_open, NULL);
+	break;
+      case SWFDEC_LOADER_STATE_READING:
+	swfdec_player_add_external_action (loader->player, loader,
+	    swfdec_loader_perform_open, NULL);
+	swfdec_player_add_external_action (loader->player, loader,
+	    swfdec_loader_perform_push, NULL);
+	break;
+      case SWFDEC_LOADER_STATE_EOF:
+	swfdec_player_add_external_action (loader->player, loader,
+	    swfdec_loader_perform_open, NULL);
+	swfdec_player_add_external_action (loader->player, loader,
+	    swfdec_loader_perform_push, NULL);
+	swfdec_player_add_external_action (loader->player, loader,
+	    swfdec_loader_perform_eof, NULL);
+	break;
+      case SWFDEC_LOADER_STATE_ERROR:
+	swfdec_player_add_external_action (loader->player, loader,
+	    swfdec_loader_perform_error, NULL);
+	break;
+      default:
+	g_assert_not_reached ();
+	break;
+    }
+  } else {
+    loader->player = NULL;
+  }
 }
 
 /** PUBLIC API ***/
 
-/**
- * swfdec_loader_new_from_file:
- * @filename: name of the file to load
- *
- * Creates a new loader for local files. If an error occurred, the loader will
- * be in error.
- *
- * Returns: a new loader
- **/
-SwfdecLoader *
-swfdec_loader_new_from_file (const char *filename)
-{
-  SwfdecBuffer *buf;
-  SwfdecLoader *loader;
-  GError *error = NULL;
-
-  g_return_val_if_fail (filename != NULL, NULL);
-
-  buf = swfdec_buffer_new_from_file (filename, &error);
-
-  loader = g_object_new (SWFDEC_TYPE_FILE_LOADER, NULL);
-  if (g_path_is_absolute (filename)) {
-    loader->url = g_strdup (filename);
-  } else {
-    char *cur = g_get_current_dir ();
-    loader->url = g_build_filename (cur, filename, NULL);
-    g_free (cur);
-  }
-  SWFDEC_FILE_LOADER (loader)->dir = g_path_get_dirname (loader->url);
-  if (buf == NULL) {
-    swfdec_loader_error (loader, error->message);
-    g_error_free (error);
-  } else {
-    swfdec_loader_set_size (loader, buf->length);
-    swfdec_loader_push (loader, buf);
-    swfdec_loader_eof (loader);
-  }
-  return loader;
-}
-
 /**
  * swfdec_loader_error:
  * @loader: a #SwfdecLoader
@@ -343,8 +318,6 @@ swfdec_loader_new_from_file (const char *filename)
 void
 swfdec_loader_error (SwfdecLoader *loader, const char *error)
 {
-  SwfdecPlayer *player;
-
   g_return_if_fail (SWFDEC_IS_LOADER (loader));
   g_return_if_fail (error != NULL);
 
@@ -353,44 +326,37 @@ swfdec_loader_error (SwfdecLoader *loader, const char *error)
     return;
   }
 
-  if (loader->target) {
-    player = swfdec_loader_target_get_player (loader->target);
-    swfdec_player_lock (player);
-    swfdec_loader_error_locked (loader, error);
-    swfdec_player_perform_actions (player);
-    swfdec_player_unlock (player);
-  } else {
-    swfdec_loader_error_locked (loader, error);
-  }
-}
-
-void
-swfdec_loader_error_locked (SwfdecLoader *loader, const char *error)
-{
-  if (loader->error)
-    return;
-
-  SWFDEC_WARNING ("error in %s %p: %s", G_OBJECT_TYPE_NAME (loader), loader, error);
+  loader->state = SWFDEC_LOADER_STATE_ERROR;
   loader->error = g_strdup (error);
-  g_object_notify (G_OBJECT (loader), "error");
   if (loader->target)
-    swfdec_loader_target_parse (loader->target, loader);
+    swfdec_player_add_external_action (loader->player, loader,
+	swfdec_loader_perform_error, NULL);
 }
 
+/**
+ * swfdec_loader_open:
+ * @loader: a #SwfdecLoader
+ * @url: the real URL used for this loader if it has changed (e.g. after HTTP 
+ *       redirects) or %NULL if it hasn't changed
+ *
+ * Call this function when your loader opened the resulting file. For HTTP this
+ * is when having received the headers. You must call this function before 
+ * swfdec_laoder_push() can be called.
+ **/
 void
-swfdec_loader_parse (SwfdecLoader *loader)
+swfdec_loader_open (SwfdecLoader *loader, const char *url)
 {
-  SwfdecPlayer *player;
+  g_return_if_fail (SWFDEC_IS_LOADER (loader));
+  g_return_if_fail (loader->state == SWFDEC_LOADER_STATE_NEW);
 
-  if (loader->target == NULL ||
-      loader->error)
-    return;
-
-  player = swfdec_loader_target_get_player (loader->target);
-  swfdec_player_lock (player);
-  swfdec_loader_target_parse (loader->target, loader);
-  swfdec_player_perform_actions (player);
-  swfdec_player_unlock (player);
+  loader->state = SWFDEC_LOADER_STATE_OPEN;
+  if (url) {
+    swfdec_url_free (loader->url);
+    loader->url = swfdec_url_new (url);
+    g_object_notify (G_OBJECT (loader), "url");
+  }
+  if (loader->player)
+    swfdec_player_add_external_action (loader->player, loader, swfdec_loader_perform_open, NULL);
 }
 
 /**
@@ -399,40 +365,46 @@ swfdec_loader_parse (SwfdecLoader *loader)
  * @buffer: new data to make available. The loader takes the reference
  *          to the buffer.
  *
- * Makes the data in @buffer available to @loader and processes it.
+ * Makes the data in @buffer available to @loader and processes it. The @loader
+ * must be open.
  **/
 void
 swfdec_loader_push (SwfdecLoader *loader, SwfdecBuffer *buffer)
 {
   g_return_if_fail (SWFDEC_IS_LOADER (loader));
-  g_return_if_fail (loader->eof == FALSE);
+  g_return_if_fail (loader->state == SWFDEC_LOADER_STATE_OPEN || loader->state == SWFDEC_LOADER_STATE_READING);
   g_return_if_fail (buffer != NULL);
 
   swfdec_buffer_queue_push (loader->queue, buffer);
   g_object_notify (G_OBJECT (loader), "loaded");
-  swfdec_loader_parse (loader);
+  loader->state = SWFDEC_LOADER_STATE_READING;
+  if (loader->player)
+    swfdec_player_add_external_action (loader->player, loader,
+	swfdec_loader_perform_push, NULL);
 }
 
 /**
  * swfdec_loader_eof:
  * @loader: a #SwfdecLoader
  *
- * Indicates to @loader that no more data will follow.
+ * Indicates to @loader that no more data will follow. The loader must be open.
  **/
 void
 swfdec_loader_eof (SwfdecLoader *loader)
 {
   g_return_if_fail (SWFDEC_IS_LOADER (loader));
-  g_return_if_fail (loader->eof == FALSE);
+  g_return_if_fail (loader->state == SWFDEC_LOADER_STATE_OPEN || loader->state == SWFDEC_LOADER_STATE_READING);
 
-  loader->eof = TRUE;
   if (loader->size == 0) {
     gulong bytes = swfdec_loader_get_loaded (loader);
     if (bytes)
       swfdec_loader_set_size (loader, bytes);
   }
   g_object_notify (G_OBJECT (loader), "eof");
-  swfdec_loader_parse (loader);
+  loader->state = SWFDEC_LOADER_STATE_EOF;
+  if (loader->player)
+    swfdec_player_add_external_action (loader->player, loader,
+	swfdec_loader_perform_eof, NULL);
 }
 
 /**
@@ -448,49 +420,58 @@ swfdec_loader_eof (SwfdecLoader *loader)
 char *
 swfdec_loader_get_filename (SwfdecLoader *loader)
 {
-  char *start, *end, *ret;
+  const SwfdecURL *url;
+  const char *path, *ext;
+  char *ret = NULL;
 
   g_return_val_if_fail (SWFDEC_IS_LOADER (loader), NULL);
-  /* every loader must set this */
-  g_return_val_if_fail (loader->url != NULL, NULL);
 
-  end = strchr (loader->url, '?');
-  if (end) {
-    char *next = NULL;
-    do {
-      start = next ? next + 1 : loader->url;
-      next = strchr (start, '/');
-    } while (next != NULL && next < end);
-  } else {
-    start = strrchr (loader->url, '/');
-    if (start == NULL) {
-      start = loader->url;
-    } else {
-      start++;
-    }
+  url = swfdec_loader_get_url (loader);
+  path = swfdec_url_get_path (url);
+  if (path) {
+    char *s = strchr (path, '/');
+    if (s)
+      path = s + 1;
+    if (path[0] == 0)
+      path = NULL;
   }
-  ret = g_filename_from_utf8 (start, end ? end - start : -1, NULL, NULL, NULL);
-  if (ret) {
-    const char *ext;
-    
-    ext = swfdec_loader_data_type_get_extension (loader->data_type);
-    if (*ext) {
-      char *dot = strrchr (ret, '.');
-      char *real;
-      guint len = dot ? strlen (dot) : G_MAXUINT;
-      if (len <= 5)
-	*dot = '\0';
-      real = g_strdup_printf ("%s.%s", ret, ext);
-      g_free (ret);
-      ret = real;
-    }
-  } else {
-    ret = g_strdup ("unknown file");
+  if (path)
+    ret = g_filename_from_utf8 (path, -1, NULL, NULL, NULL);
+  if (ret == NULL)
+    ret = g_strdup ("unknown");
+
+  ext = swfdec_loader_data_type_get_extension (loader->data_type);
+  if (*ext) {
+    char *dot = strrchr (ret, '.');
+    char *real;
+    guint len = dot ? strlen (dot) : G_MAXUINT;
+    if (len <= 5)
+      *dot = '\0';
+    real = g_strdup_printf ("%s.%s", ret, ext);
+    g_free (ret);
+    ret = real;
   }
 
   return ret;
 }
 
+/**
+ * swfdec_loader_get_url:
+ * @loader: a #SwfdecLoader
+ *
+ * Gets the url this loader is handling. This is mostly useful for writing 
+ * subclasses of #SwfdecLoader.
+ *
+ * Returns: a #SwfdecURL describing @loader.
+ **/
+const SwfdecURL *
+swfdec_loader_get_url (SwfdecLoader *loader)
+{
+  g_return_val_if_fail (SWFDEC_IS_LOADER (loader), NULL);
+
+  return loader->url;
+}
+
 /**
  * swfdec_loader_get_data_type:
  * @loader: a #SwfdecLoader
@@ -605,7 +586,7 @@ swfdec_loader_data_type_get_extension (SwfdecLoaderDataType type)
 /*** X-WWW-FORM-URLENCODED ***/
 
 /* if speed ever gets an issue, use a 256 byte array instead of strchr */
-static const char *urlencode_unescaped="0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz-_.:/()'";
+static const char *urlencode_unescaped="0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz-_.,:/()'";
 static void
 swfdec_urlencode_append_string (GString *str, const char *s)
 {
@@ -679,7 +660,7 @@ swfdec_urldecode_one_string (const char *s, const char **out)
  * to the given @str
  **/
 void
-swfdec_string_append_urlencoded (GString *str, char *name, char *value)
+swfdec_string_append_urlencoded (GString *str, const char *name, const char *value)
 {
   g_return_if_fail (str != NULL);
   g_return_if_fail (name != NULL);
