@@ -20,40 +20,44 @@
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
-#include <string.h>
-#include <math.h>
 
-#include <js/jsapi.h>
+#include <math.h>
+#include <stdlib.h>
+#include <string.h>
+#include <strings.h>
+#include <errno.h>
 
 #include "swfdec_movie.h"
+#include "swfdec_as_context.h"
+#include "swfdec_as_strings.h"
+#include "swfdec_button_movie.h"
 #include "swfdec_debug.h"
 #include "swfdec_debugger.h"
 #include "swfdec_event.h"
 #include "swfdec_graphic.h"
-#include "swfdec_js.h"
+#include "swfdec_loader_internal.h"
 #include "swfdec_player_internal.h"
-#include "swfdec_root_movie.h"
 #include "swfdec_sprite.h"
+#include "swfdec_sprite_movie.h"
+#include "swfdec_swf_instance.h"
 
 /*** MOVIE ***/
 
-static const SwfdecContent default_content = SWFDEC_CONTENT_DEFAULT;
-
-G_DEFINE_ABSTRACT_TYPE (SwfdecMovie, swfdec_movie, SWFDEC_TYPE_SCRIPTABLE)
+G_DEFINE_ABSTRACT_TYPE (SwfdecMovie, swfdec_movie, SWFDEC_TYPE_AS_OBJECT)
 
 static void
 swfdec_movie_init (SwfdecMovie * movie)
 {
-  movie->content = &default_content;
-
   movie->xscale = 100;
   movie->yscale = 100;
+  cairo_matrix_init_identity (&movie->original_transform);
   cairo_matrix_init_identity (&movie->matrix);
   cairo_matrix_init_identity (&movie->inverse_matrix);
+
   swfdec_color_transform_init_identity (&movie->color_transform);
+  swfdec_color_transform_init_identity (&movie->original_ctrans);
 
   movie->visible = TRUE;
-  movie->n_frames = 1;
 
   swfdec_rect_init_empty (&movie->extents);
 }
@@ -79,7 +83,7 @@ swfdec_movie_invalidate (SwfdecMovie *movie)
       return;
     swfdec_rect_transform (&rect, &rect, &movie->matrix);
   }
-  swfdec_player_invalidate (SWFDEC_ROOT_MOVIE (movie)->player, &rect);
+  swfdec_player_invalidate (SWFDEC_PLAYER (SWFDEC_AS_OBJECT (movie)->context), &rect);
 }
 
 /**
@@ -90,7 +94,7 @@ swfdec_movie_invalidate (SwfdecMovie *movie)
  * Queues an update of all cached values inside @movie and invalidates it.
  **/
 void
-swfdec_movie_queue_update (SwfdecMovie *movie, SwfdecMovieState state)
+swfdec_movie_queue_update (SwfdecMovie *movie, SwfdecMovieCacheState state)
 {
   g_return_if_fail (SWFDEC_IS_MOVIE (movie));
 
@@ -135,16 +139,23 @@ swfdec_movie_update_matrix (SwfdecMovie *movie)
 {
   double d, e;
 
-  movie->matrix.xx = movie->content->transform.xx;
-  movie->matrix.xy = movie->content->transform.xy;
-  movie->matrix.yx = movie->content->transform.yx;
-  movie->matrix.yy = movie->content->transform.yy;
+  /* we operate on x0 and y0 when setting movie._x and movie._y */
+  if (movie->modified) {
+    movie->matrix.xx = movie->original_transform.xx;
+    movie->matrix.yx = movie->original_transform.yx;
+    movie->matrix.xy = movie->original_transform.xy;
+    movie->matrix.yy = movie->original_transform.yy;
+  } else {
+    movie->matrix = movie->original_transform;
+  }
 
-  d = movie->xscale / swfdec_matrix_get_xscale (&movie->content->transform);
-  e = movie->yscale / swfdec_matrix_get_yscale (&movie->content->transform);
+  d = movie->xscale / swfdec_matrix_get_xscale (&movie->original_transform);
+  e = movie->yscale / swfdec_matrix_get_yscale (&movie->original_transform);
   cairo_matrix_scale (&movie->matrix, d, e);
-  d = movie->rotation - swfdec_matrix_get_rotation (&movie->content->transform);
-  cairo_matrix_rotate (&movie->matrix, d * G_PI / 180);
+  if (isfinite (movie->rotation)) {
+    d = movie->rotation - swfdec_matrix_get_rotation (&movie->original_transform);
+    cairo_matrix_rotate (&movie->matrix, d * G_PI / 180);
+  }
   swfdec_matrix_ensure_invertible (&movie->matrix, &movie->inverse_matrix);
 
   swfdec_movie_update_extents (movie);
@@ -203,53 +214,6 @@ swfdec_movie_update (SwfdecMovie *movie)
   }
 }
 
-/**
- * swfdec_movie_set_content:
- * @movie: a #SwfdecMovie
- * @content: #SwfdecContent to set for this movie or NULL to unset
- *
- * Sets new contents for @movie. Note that name and graphic of @content must 
- * be identical to the current content of @movie.
- **/
-void
-swfdec_movie_set_content (SwfdecMovie *movie, const SwfdecContent *content)
-{
-  const SwfdecContent *old_content;
-  SwfdecMovieClass *klass;
-
-  g_return_if_fail (SWFDEC_IS_MOVIE (movie));
-
-  if (movie->content == content)
-    return;
-
-  if (content == NULL) {
-    content = &default_content;
-  } else if (movie->content != &default_content) {
-    g_return_if_fail (movie->depth == content->depth);
-    g_return_if_fail (movie->content->graphic == content->graphic);
-    if (content->name) {
-      g_return_if_fail (movie->content->name != NULL);
-      g_return_if_fail (g_str_equal (content->name, movie->content->name));
-    } else {
-      g_return_if_fail (movie->content->name == NULL);
-    }
-  }
-  SWFDEC_LOG ("setting content of movie %s from %p to %p", 
-      movie->name, movie->content, content);
-  old_content = movie->content;
-  klass = SWFDEC_MOVIE_GET_CLASS (movie);
-  if (klass->content_changed)
-    klass->content_changed (movie, content);
-  movie->content = content;
-  if (!movie->modified) {
-    movie->matrix = content->transform;
-    movie->xscale = swfdec_matrix_get_xscale (&movie->matrix);
-    movie->yscale = swfdec_matrix_get_yscale (&movie->matrix);
-    movie->rotation = swfdec_matrix_get_rotation (&movie->matrix);
-    swfdec_movie_queue_update (movie, SWFDEC_MOVIE_INVALID_MATRIX);
-  }
-}
-
 SwfdecMovie *
 swfdec_movie_find (SwfdecMovie *movie, int depth)
 {
@@ -258,51 +222,71 @@ swfdec_movie_find (SwfdecMovie *movie, int depth)
   g_return_val_if_fail (SWFDEC_IS_MOVIE (movie), NULL);
 
   for (walk = movie->list; walk; walk = walk->next) {
-    SwfdecMovie *movie = walk->data;
+    SwfdecMovie *cur= walk->data;
 
-    if (movie->depth < depth)
+    if (cur->depth < depth)
       continue;
-    if (movie->depth == depth)
-      return movie;
+    if (cur->depth == depth)
+      return cur;
     break;
   }
   return NULL;
 }
 
-typedef void (* SwfdecMovieRemoveFunc) (SwfdecMovie *, gpointer);
-
-static void
-swfdec_movie_do_remove (SwfdecMovie *movie, gpointer child_remove)
+static gboolean
+swfdec_movie_do_remove (SwfdecMovie *movie)
 {
+  SwfdecPlayer *player;
+
+  SWFDEC_LOG ("removing %s %s", G_OBJECT_TYPE_NAME (movie), movie->name);
+
+  player = SWFDEC_PLAYER (SWFDEC_AS_OBJECT (movie)->context);
   movie->will_be_removed = TRUE;
-  /* remove all children */
   while (movie->list) {
-    (*(SwfdecMovieRemoveFunc) child_remove) (movie->list->data, child_remove);
+    GList *walk = movie->list;
+    while (walk && SWFDEC_MOVIE (walk->data)->will_be_removed)
+      walk = walk->next;
+    if (walk == NULL)
+      break;
+    swfdec_movie_remove (walk->data);
   }
-  if (SWFDEC_ROOT_MOVIE (movie->root)->player->mouse_grab == movie)
-    SWFDEC_ROOT_MOVIE (movie->root)->player->mouse_grab = NULL;
-  if (SWFDEC_ROOT_MOVIE (movie->root)->player->mouse_drag == movie)
-    SWFDEC_ROOT_MOVIE (movie->root)->player->mouse_drag = NULL;
+  /* FIXME: all of this here or in destroy callback? */
+  if (player->mouse_grab == movie)
+    player->mouse_grab = NULL;
+  if (player->mouse_drag == movie)
+    player->mouse_drag = NULL;
   swfdec_movie_invalidate (movie);
-  if (movie->parent) {
-    SwfdecPlayer *player = SWFDEC_ROOT_MOVIE (movie->root)->player;
-    if (SWFDEC_IS_DEBUGGER (player) &&
-	g_list_find (movie->parent->list, movie)) {
-      movie->parent->list = g_list_remove (movie->parent->list, movie);
-      g_signal_emit_by_name (player, "movie-removed", movie);
-    } else {
-      movie->parent->list = g_list_remove (movie->parent->list, movie);
-    }
-  } else {
-    SwfdecPlayer *player = SWFDEC_ROOT_MOVIE (movie)->player;
-    if (SWFDEC_IS_DEBUGGER (player) &&
-	g_list_find (player->roots, movie)) {
-      player->roots = g_list_remove (player->roots, movie);
-      g_signal_emit_by_name (player, "movie-removed", movie);
-    } else {
-      player->roots = g_list_remove (player->roots, movie);
-    }
-  }
+  movie->depth = -16385 - movie->depth; /* don't ask me why... */
+  if (movie->parent)
+    movie->parent->list = g_list_sort (movie->parent->list, swfdec_movie_compare_depths);
+
+  if (SWFDEC_IS_SPRITE_MOVIE (movie))
+    return !swfdec_movie_queue_script (movie, SWFDEC_EVENT_UNLOAD);
+  else
+    return TRUE;
+}
+
+/**
+ * swfdec_movie_remove:
+ * @movie: #SwfdecMovie to remove
+ *
+ * Removes this movie from its parent. In contrast to swfdec_movie_destroy (),
+ * it will definitely cause a removal from the display list, but depending on
+ * movie, it might still be possible to reference it from Actionscript.
+ **/
+void
+swfdec_movie_remove (SwfdecMovie *movie)
+{
+  gboolean result;
+
+  g_return_if_fail (SWFDEC_IS_MOVIE (movie));
+
+  if (movie->state > SWFDEC_MOVIE_STATE_RUNNING)
+    return;
+  result = swfdec_movie_do_remove (movie);
+  movie->state = SWFDEC_MOVIE_STATE_REMOVED;
+  if (result)
+    swfdec_movie_destroy (movie);
 }
 
 /**
@@ -317,39 +301,80 @@ void
 swfdec_movie_destroy (SwfdecMovie *movie)
 {
   SwfdecMovieClass *klass = SWFDEC_MOVIE_GET_CLASS (movie);
-  SwfdecPlayer *player = SWFDEC_ROOT_MOVIE (movie->root)->player;
+  SwfdecPlayer *player = SWFDEC_PLAYER (SWFDEC_AS_OBJECT (movie)->context);
 
+  g_assert (movie->state < SWFDEC_MOVIE_STATE_DESTROYED);
+  if (movie->state < SWFDEC_MOVIE_STATE_REMOVED) {
+    swfdec_movie_do_remove (movie);
+  }
   SWFDEC_LOG ("destroying movie %s", movie->name);
-  swfdec_movie_do_remove (movie, swfdec_movie_destroy);
-  swfdec_movie_set_content (movie, NULL);
+  while (movie->list) {
+    swfdec_movie_destroy (movie->list->data);
+  }
+  if (movie->parent) {
+    if (SWFDEC_IS_DEBUGGER (player) &&
+	g_list_find (movie->parent->list, movie)) {
+      g_signal_emit_by_name (player, "movie-removed", movie);
+    }
+    movie->parent->list = g_list_remove (movie->parent->list, movie);
+  } else {
+    if (SWFDEC_IS_DEBUGGER (player) &&
+	g_list_find (player->roots, movie)) {
+      g_signal_emit_by_name (player, "movie-removed", movie);
+    }
+    player->roots = g_list_remove (player->roots, movie);
+  }
   /* FIXME: figure out how to handle destruction pre-init/construct.
    * This is just a stop-gap measure to avoid dead movies in those queues */
   g_queue_remove (player->init_queue, movie);
   g_queue_remove (player->construct_queue, movie);
+  swfdec_player_remove_all_actions (player, movie);
   if (klass->finish_movie)
     klass->finish_movie (movie);
-  swfdec_js_movie_remove_jsobject (movie);
   player->movies = g_list_remove (player->movies, movie);
+  movie->state = SWFDEC_MOVIE_STATE_DESTROYED;
+  /* unset prototype here, so we don't work in AS anymore */
+  SWFDEC_AS_OBJECT (movie)->prototype = NULL;
   g_object_unref (movie);
 }
 
 /**
- * swfdec_movie_remove:
- * @movie: #SwfdecMovie to remove
+ * swfdec_movie_run_init:
+ * @movie: a #SwfdecMovie
  *
- * Removes this movie from its parent. In contrast to swfdec_movie_destroy (),
- * it will definitely cause a removal from the display list, but depending on
- * movie, it might still be possible to reference it from Actionscript.
- **/
+ * Runs onClipEvent(initialize) on the given @movie.
+ */
 void
-swfdec_movie_remove (SwfdecMovie *movie)
+swfdec_movie_run_init (SwfdecMovie *movie)
 {
+  SwfdecPlayer *player;
+
   g_return_if_fail (SWFDEC_IS_MOVIE (movie));
 
-  SWFDEC_LOG ("removing %s %s", G_OBJECT_TYPE_NAME (movie), movie->name);
-  swfdec_movie_do_remove (movie, swfdec_movie_remove);
-  if (!swfdec_movie_queue_script (movie, SWFDEC_EVENT_UNLOAD))
-    swfdec_movie_destroy (movie);
+  player = SWFDEC_PLAYER (SWFDEC_AS_OBJECT (movie)->context);
+  g_queue_remove (player->init_queue, movie);
+  swfdec_movie_execute_script (movie, SWFDEC_EVENT_INITIALIZE);
+}
+
+/**
+ * swfdec_movie_run_construct:
+ * @movie: a #SwfdecMovie
+ *
+ * Runs the constructors for @movie. This is (in the given order) 
+ * onClipEvent(construct), movie.onConstruct and the constructor registered
+ * via Object.registerClass.
+ **/
+void
+swfdec_movie_run_construct (SwfdecMovie *movie)
+{
+  SwfdecPlayer *player;
+
+  g_return_if_fail (SWFDEC_IS_MOVIE (movie));
+
+  player = SWFDEC_PLAYER (SWFDEC_AS_OBJECT (movie)->context);
+  g_queue_remove (player->construct_queue, movie);
+  swfdec_movie_execute_script (movie, SWFDEC_EVENT_CONSTRUCT);
+  swfdec_as_object_call (SWFDEC_AS_OBJECT (movie), SWFDEC_AS_STR___constructor__, 0, NULL, NULL);
 }
 
 void
@@ -360,13 +385,13 @@ swfdec_movie_execute_script (SwfdecMovie *movie, SwfdecEventType condition)
   g_return_if_fail (SWFDEC_IS_MOVIE (movie));
   g_return_if_fail (condition != 0);
 
-  if (movie->content->events) {
-    swfdec_event_list_execute (movie->content->events, 
-	SWFDEC_SCRIPTABLE (movie), condition, 0);
+  if (movie->events) {
+    swfdec_event_list_execute (movie->events, 
+	SWFDEC_AS_OBJECT (movie), condition, 0);
   }
   name = swfdec_event_type_get_name (condition);
   if (name != NULL)
-    swfdec_scriptable_execute (SWFDEC_SCRIPTABLE (movie), name, 0, NULL);
+    swfdec_as_object_call (SWFDEC_AS_OBJECT (movie), name, 0, NULL, NULL);
 }
 
 static void
@@ -392,23 +417,69 @@ swfdec_movie_queue_script (SwfdecMovie *movie, SwfdecEventType condition)
   g_return_val_if_fail (SWFDEC_IS_MOVIE (movie), FALSE);
   g_return_val_if_fail (condition != 0, FALSE);
 
-  if (movie->content->events) {
-    if (!swfdec_event_list_has_conditions (movie->content->events, 
-	  SWFDEC_SCRIPTABLE (movie), condition, 0))
+  if (movie->events) {
+    if (!swfdec_event_list_has_conditions (movie->events, 
+	  SWFDEC_AS_OBJECT (movie), condition, 0))
       return FALSE;
   } else {
     const char *name = swfdec_event_type_get_name (condition);
     if (name == NULL ||
-	!swfdec_scriptable_can_execute (SWFDEC_SCRIPTABLE (movie), name))
+	!swfdec_as_object_has_function (SWFDEC_AS_OBJECT (movie), name))
       return FALSE;
   }
 
-  player = SWFDEC_ROOT_MOVIE (movie->root)->player;
+  player = SWFDEC_PLAYER (SWFDEC_AS_OBJECT (movie)->context);
   swfdec_player_add_action (player, movie, swfdec_movie_do_execute_script, 
       GUINT_TO_POINTER (condition));
   return TRUE;
 }
 
+/**
+ * swfdec_movie_set_variables:
+ * @script: a #SwfdecMovie
+ * @variables: variables to set on @movie in application-x-www-form-urlencoded 
+ *             format
+ * 
+ * Verifies @variables to be encoded correctly and sets them as string 
+ * properties on the given @movie.
+ **/
+void
+swfdec_movie_set_variables (SwfdecMovie *movie, const char *variables)
+{
+  SwfdecAsObject *as;
+
+  g_return_if_fail (SWFDEC_IS_MOVIE (movie));
+  g_return_if_fail (variables != NULL);
+
+  as = SWFDEC_AS_OBJECT (movie);
+  SWFDEC_DEBUG ("setting variables on %p: %s", movie, variables);
+  while (TRUE) {
+    char *name, *value;
+    const char *asname;
+    SwfdecAsValue val;
+
+    while (*variables == '&')
+      variables++;
+    if (*variables == '\0')
+      break;
+    if (!swfdec_urldecode_one (variables, &name, &value, &variables)) {
+      SWFDEC_WARNING ("variables invalid at \"%s\"", variables);
+      break;
+    }
+    if (*variables != '\0' && *variables != '&') {
+      SWFDEC_WARNING ("variables not delimited with & at \"%s\"", variables);
+      g_free (name);
+      g_free (value);
+      break;
+    }
+    asname = swfdec_as_context_give_string (as->context, name);
+    SWFDEC_AS_VALUE_SET_STRING (&val, swfdec_as_context_get_string (as->context, value));
+    g_free (value);
+    swfdec_as_object_set_variable (as, asname, &val);
+    SWFDEC_LOG ("Set variable \"%s\" to \"%s\"", name, value);
+  }
+}
+
 /* NB: coordinates are in movie's coordiante system. Use swfdec_movie_get_mouse
  * if you have global coordinates */
 static gboolean
@@ -425,16 +496,30 @@ swfdec_movie_mouse_in (SwfdecMovie *movie, double x, double y)
 void
 swfdec_movie_local_to_global (SwfdecMovie *movie, double *x, double *y)
 {
+  SwfdecPlayer *player = SWFDEC_PLAYER (SWFDEC_AS_OBJECT (movie)->context);
+
   do {
     cairo_matrix_transform_point (&movie->matrix, x, y);
   } while ((movie = movie->parent));
+
+  *x /= player->scale_x;
+  *y /= player->scale_y;
+  *x += SWFDEC_TWIPS_TO_DOUBLE (player->offset_x);
+  *y += SWFDEC_TWIPS_TO_DOUBLE (player->offset_y);
 }
 
 void
 swfdec_movie_global_to_local (SwfdecMovie *movie, double *x, double *y)
 {
-  if (movie->parent)
+  if (movie->parent) {
     swfdec_movie_global_to_local (movie->parent, x, y);
+  } else {
+    SwfdecPlayer *player = SWFDEC_PLAYER (SWFDEC_AS_OBJECT (movie)->context);
+    *x -= SWFDEC_TWIPS_TO_DOUBLE (player->offset_x);
+    *y -= SWFDEC_TWIPS_TO_DOUBLE (player->offset_y);
+    *x *= player->scale_x;
+    *y *= player->scale_y;
+  }
   if (movie->cache_state >= SWFDEC_MOVIE_INVALID_MATRIX)
     swfdec_movie_update (movie);
   cairo_matrix_transform_point (&movie->inverse_matrix, x, y);
@@ -457,7 +542,7 @@ swfdec_movie_get_mouse (SwfdecMovie *movie, double *x, double *y)
   g_return_if_fail (x != NULL);
   g_return_if_fail (y != NULL);
 
-  player = SWFDEC_ROOT_MOVIE (movie->root)->player;
+  player = SWFDEC_PLAYER (SWFDEC_AS_OBJECT (movie)->context);
   *x = player->mouse_x;
   *y = player->mouse_y;
   swfdec_movie_global_to_local (movie, x, y);
@@ -477,7 +562,7 @@ swfdec_movie_send_mouse_change (SwfdecMovie *movie, gboolean release)
     button = 0;
   } else {
     mouse_in = swfdec_movie_mouse_in (movie, x, y);
-    button = SWFDEC_ROOT_MOVIE (movie->root)->player->mouse_button;
+    button = SWFDEC_PLAYER (SWFDEC_AS_OBJECT (movie)->context)->mouse_button;
   }
   klass = SWFDEC_MOVIE_GET_CLASS (movie);
   g_assert (klass->mouse_change != NULL);
@@ -514,18 +599,18 @@ swfdec_movie_get_movie_at (SwfdecMovie *movie, double x, double y)
       clip_depth = 0;
       for (clip_walk = clip_walk->prev; clip_walk; clip_walk = clip_walk->prev) {
 	SwfdecMovie *clip = walk->data;
-	if (clip->content->clip_depth) {
+	if (clip->clip_depth) {
 	  double tmpx = x, tmpy = y;
 	  cairo_matrix_transform_point (&clip->inverse_matrix, &tmpx, &tmpy);
 	  if (!swfdec_movie_mouse_in (clip, tmpx, tmpy)) {
-	    SWFDEC_LOG ("skipping depth %d to %d due to clipping", clip->content->depth, clip->content->clip_depth);
-	    clip_depth = child->content->clip_depth;
+	    SWFDEC_LOG ("skipping depth %d to %d due to clipping", clip->depth, clip->clip_depth);
+	    clip_depth = child->clip_depth;
 	  }
 	  break;
 	}
       }
     }
-    if (child->content->clip_depth) {
+    if (child->clip_depth) {
       SWFDEC_LOG ("resetting clip depth");
       clip_depth = 0;
       continue;
@@ -587,13 +672,13 @@ swfdec_movie_render (SwfdecMovie *movie, cairo_t *cr,
   swfdec_rect_transform (&rect, inval, &movie->inverse_matrix);
   SWFDEC_LOG ("%sinvalid area is now: %g %g  %g %g",  movie->parent ? "  " : "",
       rect.x0, rect.y0, rect.x1, rect.y1);
-  swfdec_color_transform_chain (&trans, &movie->content->color_transform, color_transform);
+  swfdec_color_transform_chain (&trans, &movie->original_ctrans, color_transform);
   swfdec_color_transform_chain (&trans, &movie->color_transform, &trans);
 
   for (g = movie->list; g; g = g_list_next (g)) {
     SwfdecMovie *child = g->data;
 
-    if (child->content->clip_depth) {
+    if (child->clip_depth) {
       if (clip_depth) {
 	/* FIXME: is clipping additive? */
 	SWFDEC_INFO ("unsetting clip depth %d for new clip depth", clip_depth);
@@ -606,9 +691,9 @@ swfdec_movie_render (SwfdecMovie *movie, cairo_t *cr,
 	/* FIXME FIXME FIXME: overlapping objects in the clip movie cause problems
 	 * due to them being accumulated with CAIRO_FILL_RULE_EVEN_ODD
 	 */
-	SWFDEC_INFO ("clipping up to depth %d by using %p with depth %d", child->content->clip_depth,
+	SWFDEC_INFO ("clipping up to depth %d by using %p with depth %d", child->clip_depth,
 	    child, child->depth);
-	clip_depth = child->content->clip_depth;
+	clip_depth = child->clip_depth;
 	cairo_save (cr);
 	swfdec_movie_render (child, cr, &trans, &rect, FALSE);
 	cairo_clip (cr);
@@ -659,181 +744,383 @@ swfdec_movie_dispose (GObject *object)
   SwfdecMovie * movie = SWFDEC_MOVIE (object);
 
   g_assert (movie->list == NULL);
-  g_assert (movie->content == &default_content);
 
-  SWFDEC_LOG ("disposing movie %s", movie->name);
-  g_free (movie->name);
-  if (movie->parent)
-    g_object_unref (movie->parent);
+  SWFDEC_LOG ("disposing movie %s (depth %d)", movie->name, movie->depth);
+  if (movie->swf) {
+    g_object_unref (movie->swf);
+    movie->swf = NULL;
+  }
+  if (movie->events) {
+    swfdec_event_list_free (movie->events);
+    movie->events = NULL;
+  }
+  if (movie->graphic) {
+    g_object_unref (movie->graphic);
+    movie->graphic = NULL;
+  }
 
   G_OBJECT_CLASS (swfdec_movie_parent_class)->dispose (G_OBJECT (movie));
 }
 
+static void
+swfdec_movie_mark (SwfdecAsObject *object)
+{
+  SwfdecMovie *movie = SWFDEC_MOVIE (object);
+  GList *walk;
+
+  swfdec_as_string_mark (movie->name);
+  for (walk = movie->list; walk; walk = walk->next) {
+    swfdec_as_object_mark (walk->data);
+  }
+
+  SWFDEC_AS_OBJECT_CLASS (swfdec_movie_parent_class)->mark (object);
+}
+
+/* FIXME: This function can definitely be implemented easier */
+static SwfdecMovie *
+swfdec_movie_get_by_name (SwfdecPlayer *player, const char *name)
+{
+  GList *walk;
+  int i = SWFDEC_AS_CONTEXT (player)->version;
+  gulong l;
+  char *end;
+
+  if ((i >= 7 && !g_str_has_prefix (name, "_level")) ||
+      strncasecmp (name, "_level", 6) != 0)
+    return NULL;
+
+  errno = 0;
+  l = strtoul (name + 6, &end, 10);
+  if (errno != 0 || *end != 0 || l > G_MAXINT)
+    return NULL;
+  i = l - 16384;
+  for (walk = player->roots; walk; walk = walk->next) {
+    SwfdecMovie *cur = walk->data;
+    if (cur->depth < i)
+      continue;
+    if (cur->depth == i)
+      return cur;
+    break;
+  }
+  return NULL;
+}
+
+static gboolean
+swfdec_movie_get_variable (SwfdecAsObject *object, const char *variable, 
+    SwfdecAsValue *val, guint *flags)
+{
+  SwfdecMovie *movie = SWFDEC_MOVIE (object);
+
+  if (movie->state == SWFDEC_MOVIE_STATE_DESTROYED)
+    return FALSE;
+
+  if (swfdec_movie_get_asprop (movie, variable, val)) {
+    *flags = 0;
+    return TRUE;
+  }
+
+  if (SWFDEC_AS_OBJECT_CLASS (swfdec_movie_parent_class)->get (object, variable, val, flags))
+    return TRUE;
+
+  /* FIXME: check that this is correct */
+  if (object->context->version > 5 && variable == SWFDEC_AS_STR__global) {
+    SWFDEC_AS_VALUE_SET_OBJECT (val, object->context->global);
+    *flags = 0;
+    return TRUE;
+  }
+  
+  movie = swfdec_movie_get_by_name (SWFDEC_PLAYER (object->context), variable);
+  if (movie) {
+    SWFDEC_AS_VALUE_SET_OBJECT (val, SWFDEC_AS_OBJECT (movie));
+    *flags = 0;
+    return TRUE;
+  }
+  return FALSE;
+}
+
+static void
+swfdec_movie_set_variable (SwfdecAsObject *object, const char *variable, 
+    const SwfdecAsValue *val)
+{
+  SwfdecMovie *movie = SWFDEC_MOVIE (object);
+
+  if (movie->state == SWFDEC_MOVIE_STATE_DESTROYED)
+    return;
+  if (swfdec_movie_set_asprop (movie, variable, val))
+    return;
+  SWFDEC_AS_OBJECT_CLASS (swfdec_movie_parent_class)->set (object, variable, val);
+}
+
+static char *
+swfdec_movie_get_debug (SwfdecAsObject *object)
+{
+  SwfdecMovie *movie = SWFDEC_MOVIE (object);
+
+  return swfdec_movie_get_path (movie);
+}
+
 static gboolean
 swfdec_movie_iterate_end (SwfdecMovie *movie)
 {
   return movie->parent == NULL || 
-	 g_list_find (movie->parent->list, movie) != NULL;
+	 movie->state < SWFDEC_MOVIE_STATE_REMOVED;
 }
 
-static JSObject *
-swfdec_movie_create_js_object (SwfdecScriptable *script)
-{
-  /* we create the objects manually and ensure persistence */
-  g_assert_not_reached ();
-}
-
-extern const JSClass movieclip_class;
 static void
 swfdec_movie_class_init (SwfdecMovieClass * movie_class)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (movie_class);
-  SwfdecScriptableClass *script_class = SWFDEC_SCRIPTABLE_CLASS (movie_class);
+  SwfdecAsObjectClass *asobject_class = SWFDEC_AS_OBJECT_CLASS (movie_class);
 
   object_class->dispose = swfdec_movie_dispose;
 
-  script_class->jsclass = &movieclip_class;
-  script_class->create_js_object = swfdec_movie_create_js_object;
+  asobject_class->mark = swfdec_movie_mark;
+  asobject_class->get = swfdec_movie_get_variable;
+  asobject_class->set = swfdec_movie_set_variable;
+  asobject_class->debug = swfdec_movie_get_debug;
 
   movie_class->iterate_end = swfdec_movie_iterate_end;
 }
 
-static void
-swfdec_movie_set_name (SwfdecMovie *movie)
+void
+swfdec_movie_initialize (SwfdecMovie *movie)
 {
-  /* FIXME: implement this function in a smarter way, not if (IS_FOO_MOVIE (x)) */
-  g_assert (movie->name == NULL);
-  if (movie->content->name) {
-    movie->name = g_strdup (movie->content->name);
-    movie->has_name = TRUE;
-  } else if (SWFDEC_IS_SPRITE_MOVIE (movie)) {
-    /* FIXME: figure out if it's relative to root or player or something else
-     * entirely 
-     */
-    SwfdecRootMovie *root = SWFDEC_ROOT_MOVIE (movie->root);
-    movie->name = g_strdup_printf ("instance%u", ++root->unnamed_count);
-    movie->has_name = FALSE;
-  } else {
-    movie->name = g_strdup (G_OBJECT_TYPE_NAME (movie));
-    movie->has_name = FALSE;
-  }
-  SWFDEC_LOG ("created movie %s", movie->name);
-}
-
-static void
-swfdec_movie_set_parent (SwfdecMovie *movie)
-{
-  SwfdecMovie *parent = movie->parent;
-  SwfdecPlayer *player = SWFDEC_ROOT_MOVIE (movie->root)->player;
   SwfdecMovieClass *klass;
 
   g_return_if_fail (SWFDEC_IS_MOVIE (movie));
 
+  klass = SWFDEC_MOVIE_GET_CLASS (movie);
+  if (klass->init_movie)
+    klass->init_movie (movie);
+}
+
+/**
+ * swfdec_movie_new:
+ * @player: a #SwfdecPlayer
+ * @depth: depth of movie
+ * @parent: the parent movie or %NULL to make this a root movie
+ * @graphic: the graphic that is displayed by this movie or %NULL to create an 
+ *           empty movieclip
+ * @name: a garbage-collected string to be used as the name for this movie or 
+ *        %NULL for a default one.
+ *
+ * Creates a new movie #SwfdecMovie for the given properties. No movie may exist
+ * at the given @depth. The actual type of
+ * this movie depends on the @graphic parameter. The movie will be initialized 
+ * with default properties. No script execution will be scheduled. After all 
+ * properties are set, the new-movie signal will be emitted if @player is a 
+ * debugger.
+ *
+ * Returns: a new #SwfdecMovie
+ **/
+SwfdecMovie *
+swfdec_movie_new (SwfdecPlayer *player, int depth, SwfdecMovie *parent, SwfdecGraphic *graphic, const char *name)
+{
+  SwfdecMovie *movie;
+  gsize size;
+
+  g_return_val_if_fail (SWFDEC_IS_PLAYER (player), NULL);
+  g_return_val_if_fail (parent == NULL || SWFDEC_IS_MOVIE (parent), NULL);
+  g_return_val_if_fail (graphic == NULL || SWFDEC_IS_GRAPHIC (graphic), NULL);
+
+  /* create the right movie */
+  if (graphic == NULL) {
+    movie = g_object_new (SWFDEC_TYPE_SPRITE_MOVIE, NULL);
+    size = sizeof (SwfdecSpriteMovie);
+  } else {
+    SwfdecGraphicClass *klass = SWFDEC_GRAPHIC_GET_CLASS (graphic);
+    g_return_val_if_fail (klass->create_movie != NULL, NULL);
+    movie = klass->create_movie (graphic, &size);
+    movie->graphic = g_object_ref (graphic);
+  }
+  /* register it to the VM */
+  /* FIXME: It'd be nice if we'd not overuse memory here when calling this function from a script */
+  if (!swfdec_as_context_use_mem (SWFDEC_AS_CONTEXT (player), size)) {
+    size = 0;
+  }
+  g_object_ref (movie);
+  swfdec_as_object_add (SWFDEC_AS_OBJECT (movie), SWFDEC_AS_CONTEXT (player), size);
+  /* set essential properties */
+  movie->depth = depth;
+  movie->parent = parent;
   if (parent) {
+    movie->swf = g_object_ref (parent->swf);
     parent->list = g_list_insert_sorted (parent->list, movie, swfdec_movie_compare_depths);
-    SWFDEC_DEBUG ("inserting %s %p (depth %d) into %s %p", G_OBJECT_TYPE_NAME (movie), movie,
+    SWFDEC_DEBUG ("inserting %s %s (depth %d) into %s %p", G_OBJECT_TYPE_NAME (movie), movie->name,
 	movie->depth,  G_OBJECT_TYPE_NAME (parent), parent);
   } else {
     player->roots = g_list_insert_sorted (player->roots, movie, swfdec_movie_compare_depths);
   }
-  swfdec_movie_set_name (movie);
-  klass = SWFDEC_MOVIE_GET_CLASS (movie);
+  /* set its name */
+  if (name) {
+    movie->original_name = name;
+    movie->name = name;
+  } else {
+    movie->original_name = SWFDEC_AS_STR_EMPTY;
+    if (SWFDEC_IS_SPRITE_MOVIE (movie) || SWFDEC_IS_BUTTON_MOVIE (movie)) {
+      movie->name = swfdec_as_context_give_string (SWFDEC_AS_CONTEXT (player), 
+	  g_strdup_printf ("instance%u", ++player->unnamed_count));
+    } else {
+      movie->name = SWFDEC_AS_STR_EMPTY;
+    }
+  }
+  /* add the movie to the global movies list */
   /* NB: adding to the movies list happens before setting the parent.
    * Setting the parent does a gotoAndPlay(0) for Sprites which can cause
    * new movies to be created (and added to this list)
    */
   player->movies = g_list_prepend (player->movies, movie);
-  /* we have to create the JSObject here to get actions queued before init_movie executes */
-  swfdec_js_movie_create_jsobject (movie);
-  /* queue init and construct events for non-root movies */
-  if (movie != movie->root) {
-    g_queue_push_tail (player->init_queue, movie);
-    g_queue_push_tail (player->construct_queue, movie);
-  }
+  /* emit the new-movie signal */
   if (SWFDEC_IS_DEBUGGER (player))
     g_signal_emit_by_name (player, "movie-added", movie);
-  swfdec_movie_queue_script (movie, SWFDEC_EVENT_LOAD);
-  if (klass->init_movie)
-    klass->init_movie (movie);
+  return movie;
 }
 
-static void
-swfdec_movie_initialize (SwfdecMovie *movie, const SwfdecContent *content)
+/* FIXME: since this is only used in PlaceObject, wouldn't it be easier to just have
+ * swfdec_movie_update_static_properties (movie); that's notified when any of these change
+ * and let PlaceObject modify the movie directly?
+ */
+void
+swfdec_movie_set_static_properties (SwfdecMovie *movie, const cairo_matrix_t *transform,
+    const SwfdecColorTransform *ctrans, int ratio, int clip_depth, SwfdecEventList *events)
 {
-  const SwfdecContent *old;
+  g_return_if_fail (SWFDEC_IS_MOVIE (movie));
+  g_return_if_fail (clip_depth >= -16384 || clip_depth <= 0);
+  g_return_if_fail (ratio >= -1);
 
-  old = movie->content;
-  movie->content = content;
-  movie->depth = content->depth;
-  swfdec_movie_set_parent (movie);
-  movie->content = old;
-  swfdec_movie_set_content (movie, content);
+  if (movie->modified) {
+    SWFDEC_LOG ("%s has already been modified by scripts, ignoring updates", movie->name);
+    return;
+  }
+  if (transform) {
+    movie->original_transform = *transform;
+    movie->xscale = swfdec_matrix_get_xscale (&movie->original_transform);
+    movie->yscale = swfdec_matrix_get_yscale (&movie->original_transform);
+    movie->rotation = swfdec_matrix_get_rotation (&movie->original_transform);
+    swfdec_movie_queue_update (movie, SWFDEC_MOVIE_INVALID_MATRIX);
+  }
+  if (ctrans) {
+    movie->original_ctrans = *ctrans;
+    swfdec_movie_invalidate (movie);
+  }
+  if (ratio >= 0 && (guint) ratio != movie->original_ratio) {
+    movie->original_ratio = ratio;
+    swfdec_movie_queue_update (movie, SWFDEC_MOVIE_INVALID_EXTENTS);
+  }
+  if (clip_depth && clip_depth != movie->clip_depth) {
+    movie->clip_depth = clip_depth;
+    /* FIXME: is this correct? */
+    swfdec_movie_invalidate (movie->parent ? movie->parent : movie);
+  }
+  if (events) {
+    if (movie->events)
+      swfdec_event_list_free (movie->events);
+    movie->events = swfdec_event_list_copy (events);
+  }
 }
 
 /**
- * swfdec_movie_new:
- * @parent: the parent movie that will contain this movie
- * @content: the content to display
+ * swfdec_movie_duplicate:
+ * @movie: #SwfdecMovie to copy
+ * @name: garbage-collected name for the new copy
+ * @depth: depth to put this movie in
  *
- * Creates a new #SwfdecMovie as a child of @parent with the given content.
- * @parent must not contain a movie clip at the depth specified by @content.
+ * Creates a duplicate of @movie. The duplicate will not be initialized or
+ * queued up for any events. You have to do this manually. In particular calling
+ * swfdec_movie_initialize() on the returned movie must be done.
  *
- * Returns: a new #SwfdecMovie
+ * Returns: a newly created movie or %NULL on error
  **/
 SwfdecMovie *
-swfdec_movie_new (SwfdecMovie *parent, const SwfdecContent *content)
+swfdec_movie_duplicate (SwfdecMovie *movie, const char *name, int depth)
 {
-  SwfdecGraphicClass *klass;
-  SwfdecMovie *ret;
+  SwfdecMovie *parent, *copy;
+
+  g_return_val_if_fail (SWFDEC_IS_MOVIE (movie), NULL);
+  g_return_val_if_fail (name != NULL, NULL);
+
+  parent = movie->parent;
+  if (movie->parent == NULL) {
+    SWFDEC_FIXME ("don't know how to duplicate root movies");
+    return NULL;
+  }
+  copy = swfdec_movie_find (movie->parent, depth);
+  if (copy) {
+    SWFDEC_LOG ("depth %d already occupied while duplicating, removing old movie", depth);
+    swfdec_movie_remove (copy);
+  }
+  copy = swfdec_movie_new (SWFDEC_PLAYER (SWFDEC_AS_OBJECT (movie)->context), depth, 
+      parent, movie->graphic, name);
+  if (copy == NULL)
+    return NULL;
+  swfdec_movie_set_static_properties (copy, &movie->original_transform,
+      &movie->original_ctrans, movie->original_ratio, movie->clip_depth, movie->events);
+  return copy;
+}
+
+SwfdecMovie *
+swfdec_movie_new_for_content (SwfdecMovie *parent, const SwfdecContent *content)
+{
+  SwfdecPlayer *player;
+  SwfdecMovie *movie;
 
   g_return_val_if_fail (SWFDEC_IS_MOVIE (parent), NULL);
   g_return_val_if_fail (SWFDEC_IS_GRAPHIC (content->graphic), NULL);
   g_return_val_if_fail (swfdec_movie_find (parent, content->depth) == NULL, NULL);
 
   SWFDEC_DEBUG ("new movie for parent %p", parent);
-  klass = SWFDEC_GRAPHIC_GET_CLASS (content->graphic);
-  g_return_val_if_fail (klass->create_movie != NULL, NULL);
-  ret = klass->create_movie (content->graphic);
-  ret->parent = parent;
-  SWFDEC_SCRIPTABLE (ret)->jscx = SWFDEC_SCRIPTABLE (parent)->jscx;
-  g_object_ref (parent);
-  ret->root = parent->root;
-  swfdec_movie_initialize (ret, content);
-  return ret;
-}
+  player = SWFDEC_PLAYER (SWFDEC_AS_OBJECT (parent)->context);
+  movie = swfdec_movie_new (player, content->depth, parent, content->graphic, 
+      content->name ? swfdec_as_context_get_string (SWFDEC_AS_CONTEXT (player), content->name) : NULL);
 
-SwfdecMovie *
-swfdec_movie_new_for_player (SwfdecPlayer *player, guint depth)
-{
-  SwfdecMovie *ret;
-  SwfdecContent *content;
+  swfdec_movie_set_static_properties (movie, content->has_transform ? &content->transform : NULL,
+      content->has_color_transform ? &content->color_transform : NULL, 
+      content->ratio, content->clip_depth, content->events);
+  if (SWFDEC_IS_SPRITE_MOVIE (movie)) {
+    g_queue_push_tail (player->init_queue, movie);
+    g_queue_push_tail (player->construct_queue, movie);
+    swfdec_movie_queue_script (movie, SWFDEC_EVENT_LOAD);
+  }
+  swfdec_movie_initialize (movie);
 
-  g_return_val_if_fail (SWFDEC_IS_PLAYER (player), NULL);
-
-  content = swfdec_content_new ((int) depth - 16384);
-  content->name = g_strdup_printf ("_level%u", depth);
-  ret = g_object_new (SWFDEC_TYPE_ROOT_MOVIE, NULL);
-  g_object_weak_ref (G_OBJECT (ret), (GWeakNotify) swfdec_content_free, content);
-  SWFDEC_ROOT_MOVIE (ret)->player = player;
-  SWFDEC_SCRIPTABLE (ret)->jscx = player->jscx;
-  ret->root = ret;
-  swfdec_movie_initialize (ret, content);
-  ret->has_name = FALSE;
-
-  return ret;
+  return movie;
 }
 
 void
-swfdec_movie_goto (SwfdecMovie *movie, guint frame)
+swfdec_movie_load (SwfdecMovie *movie, const char *url, const char *target,
+    SwfdecLoaderRequest request, const char *data, gsize data_len)
 {
-  SwfdecMovieClass *klass;
+  SwfdecPlayer *player;
 
   g_return_if_fail (SWFDEC_IS_MOVIE (movie));
-  g_return_if_fail (frame < movie->n_frames);
+  g_return_if_fail (url != NULL);
+  g_return_if_fail (target != NULL);
 
-  klass = SWFDEC_MOVIE_GET_CLASS (movie);
-  if (klass->goto_frame)
-    klass->goto_frame (movie, frame);
+  player = SWFDEC_PLAYER (SWFDEC_AS_OBJECT (movie)->context);
+  /* yay for the multiple uses of GetURL - one of the crappier Flash things */
+  if (g_str_has_prefix (target, "_level")) {
+    const char *nr = target + strlen ("_level");
+    char *end;
+    guint depth;
+
+    errno = 0;
+    depth = strtoul (nr, &end, 10);
+    if (errno == 0 && *end == '\0') {
+      if (url[0] == '\0') {
+	swfdec_player_remove_level (player, depth);
+      } else {
+	SwfdecLoader *loader = swfdec_loader_load (movie->swf->loader, url,
+	    request, data, data_len);
+	g_assert (loader);
+	swfdec_player_add_level_from_loader (player, depth, loader, NULL);
+      }
+    } else {
+      SWFDEC_ERROR ("%s does not specify a valid level", target);
+    }
+    /* FIXME: what do we do here? Is returning correct?*/
+    return;
+  }
+  swfdec_player_launch (player, url, target);
 }
 
 char *
