@@ -353,7 +353,7 @@ swfdec_shape_add_styles (SwfdecSwfDecoder * s, SwfdecShape * shape,
 }
 
 int
-tag_define_shape (SwfdecSwfDecoder * s)
+tag_define_shape (SwfdecSwfDecoder * s, guint tag)
 {
   SwfdecBits *bits = &s->b;
   SwfdecShape *shape;
@@ -377,7 +377,7 @@ tag_define_shape (SwfdecSwfDecoder * s)
 }
 
 int
-tag_define_shape_3 (SwfdecSwfDecoder * s)
+tag_define_shape_3 (SwfdecSwfDecoder * s, guint tag)
 {
   SwfdecBits *bits = &s->b;
   SwfdecShape *shape;
@@ -398,13 +398,7 @@ tag_define_shape_3 (SwfdecSwfDecoder * s)
 }
 
 int
-tag_define_shape_2 (SwfdecSwfDecoder * s)
-{
-  return tag_define_shape (s);
-}
-
-int
-tag_define_shape_4 (SwfdecSwfDecoder *s)
+tag_define_shape_4 (SwfdecSwfDecoder *s, guint tag)
 {
   SwfdecBits *bits = &s->b;
   SwfdecShape *shape;
@@ -652,8 +646,9 @@ swfdec_shape_parse_curve (SwfdecBits *bits, SubPath *path,
   int cur_x, cur_y;
   int control_x, control_y;
 
-  if (swfdec_bits_getbits (bits, 2) != 2)
+  if (swfdec_bits_getbits (bits, 2) != 2) {
     g_assert_not_reached ();
+  }
 
   n_bits = swfdec_bits_getbits (bits, 4) + 2;
 
@@ -685,8 +680,9 @@ swfdec_shape_parse_line (SwfdecBits *bits, SubPath *path,
   int general_line_flag;
   int cur_x, cur_y;
 
-  if (swfdec_bits_getbits (bits, 2) != 3)
+  if (swfdec_bits_getbits (bits, 2) != 3) {
     g_assert_not_reached ();
+  }
 
   cur_x = *x;
   cur_y = *y;
@@ -715,15 +711,16 @@ swfdec_shape_parse_line (SwfdecBits *bits, SubPath *path,
   }
 }
 
-SubPath *
+static SubPath *
 swfdec_shape_parse_change (SwfdecSwfDecoder *s, SwfdecShape *shape, GArray *path_array, SubPath *path,
     int *x, int *y, SwfdecPatternFunc parse_fill, SwfdecStrokeFunc parse_stroke)
 {
   int state_new_styles, state_line_styles, state_fill_styles1, state_fill_styles0, state_moveto;
   SwfdecBits *bits = &s->b;
 
-  if (swfdec_bits_getbit (bits) != 0)
+  if (swfdec_bits_getbit (bits) != 0) {
     g_assert_not_reached ();
+  }
 
   state_new_styles = swfdec_bits_getbit (bits);
   state_line_styles = swfdec_bits_getbit (bits);
@@ -1017,7 +1014,7 @@ error:
 }
 
 int
-tag_define_morph_shape (SwfdecSwfDecoder * s)
+tag_define_morph_shape (SwfdecSwfDecoder * s, guint tag)
 {
   SwfdecBits end_bits;
   SwfdecBits *bits = &s->b;
