@@ -38,7 +38,7 @@
 #include "swfdec_morphshape.h"
 #include "swfdec_movie.h" /* for SwfdecContent */
 #include "swfdec_pattern.h"
-#include "swfdec_root_sprite.h"
+#include "swfdec_player_internal.h"
 #include "swfdec_script.h"
 #include "swfdec_shape.h"
 #include "swfdec_sound.h"
@@ -46,14 +46,14 @@
 #include "swfdec_text.h"
 #include "swfdec_video.h"
 
-int
-tag_func_end (SwfdecSwfDecoder * s)
+static int
+tag_func_end (SwfdecSwfDecoder * s, guint tag)
 {
   return SWFDEC_STATUS_OK;
 }
 
-int
-tag_func_protect (SwfdecSwfDecoder * s)
+static int
+tag_func_protect (SwfdecSwfDecoder * s, guint tag)
 {
   if (s->protection) {
     SWFDEC_INFO ("This file is really protected.");
@@ -69,8 +69,8 @@ tag_func_protect (SwfdecSwfDecoder * s)
   return SWFDEC_STATUS_OK;
 }
 
-int
-tag_func_frame_label (SwfdecSwfDecoder * s)
+static int
+tag_func_frame_label (SwfdecSwfDecoder * s, guint tag)
 {
   SwfdecSpriteFrame *frame = &s->parse_sprite->frames[s->parse_sprite->parse_frame];
   
@@ -87,8 +87,8 @@ tag_func_frame_label (SwfdecSwfDecoder * s)
 
 /* text */
 
-static int
-define_text (SwfdecSwfDecoder * s, int rgba)
+int
+tag_func_define_text (SwfdecSwfDecoder * s, guint tag)
 {
   SwfdecBits *bits = &s->b;
   int id;
@@ -150,10 +150,10 @@ define_text (SwfdecSwfDecoder * s, int rgba)
         //printf("  font = %d\n",font);
       }
       if (has_color) {
-        if (rgba) {
-          glyph.color = swfdec_bits_get_rgba (bits);
-        } else {
+        if (tag == SWFDEC_TAG_DEFINETEXT) {
           glyph.color = swfdec_bits_get_color (bits);
+        } else {
+          glyph.color = swfdec_bits_get_rgba (bits);
         }
         //printf("  color = %08x\n",glyph.color);
       }
@@ -175,21 +175,7 @@ define_text (SwfdecSwfDecoder * s, int rgba)
 }
 
 int
-tag_func_define_text (SwfdecSwfDecoder * s)
-{
-  return define_text (s, 0);
-}
-
-int
-tag_func_define_text_2 (SwfdecSwfDecoder * s)
-{
-  return define_text (s, 1);
-}
-
-
-
-int
-tag_func_define_sprite (SwfdecSwfDecoder * s)
+tag_func_define_sprite (SwfdecSwfDecoder * s, guint define_sprite_tag)
 {
   SwfdecBits parse;
   int id;
@@ -212,7 +198,7 @@ tag_func_define_sprite (SwfdecSwfDecoder * s)
   while (swfdec_bits_left (&parse)) {
     int x;
     guint tag_len;
-    SwfdecTagFunc *func;
+    SwfdecTagFunc func;
 
     x = swfdec_bits_get_u16 (&parse);
     tag = (x >> 6) & 0x3ff;
@@ -240,7 +226,7 @@ tag_func_define_sprite (SwfdecSwfDecoder * s)
       SWFDEC_ERROR ("invalid tag %d %s during DefineSprite",
           tag, swfdec_swf_decoder_get_tag_name (tag));
     } else if (s->parse_sprite->parse_frame < s->parse_sprite->n_frames) {
-      ret = func (s);
+      ret = func (s, tag);
 
       if (swfdec_bits_left (&s->b)) {
         SWFDEC_WARNING ("early parse finish (%d bytes)", 
@@ -251,6 +237,13 @@ tag_func_define_sprite (SwfdecSwfDecoder * s)
     }
   }
 
+  /* sanity check the sprite */
+  if (s->parse_sprite->n_frames != s->parse_sprite->parse_frame) {
+    SWFDEC_INFO ("not enough frames in sprite %u (have %u, want %u), filling up with empty frames",
+	id, s->parse_sprite->parse_frame, s->parse_sprite->n_frames);
+    s->parse_sprite->parse_frame = s->parse_sprite->n_frames;
+  }
+
   s->b = parse;
   /* this assumes that no recursive DefineSprite happens and we check it doesn't */
   s->parse_sprite = s->main_sprite;
@@ -259,22 +252,6 @@ tag_func_define_sprite (SwfdecSwfDecoder * s)
   return SWFDEC_STATUS_OK;
 }
 
-int
-tag_func_do_action (SwfdecSwfDecoder * s)
-{
-  SwfdecScript *script;
-  char *name;
-
-  name = g_strdup_printf ("Sprite%u.Frame%u", SWFDEC_CHARACTER (s->parse_sprite)->id,
-      s->parse_sprite->parse_frame);
-  script = swfdec_script_new_for_player (SWFDEC_DECODER (s)->player, &s->b, name, s->version);
-  g_free (name);
-  if (script)
-    swfdec_sprite_add_action (s->parse_sprite, SWFDEC_SPRITE_ACTION_SCRIPT, script);
-
-  return SWFDEC_STATUS_OK;
-}
-
 #define CONTENT_IN_FRAME(content, frame) \
   ((content)->sequence->start <= frame && \
    (content)->sequence->end > frame)
@@ -346,8 +323,8 @@ swfdec_button_append_content (SwfdecButton *button, guint states, SwfdecContent
   }
 }
 
-int
-tag_func_define_button_2 (SwfdecSwfDecoder * s)
+static int
+tag_func_define_button_2 (SwfdecSwfDecoder * s, guint tag)
 {
   SwfdecBits *bits = &s->b;
   int id;
@@ -394,11 +371,13 @@ tag_func_define_button_2 (SwfdecSwfDecoder * s)
     content = swfdec_content_new (depth);
 
     swfdec_bits_get_matrix (bits, &content->transform, NULL);
+    content->has_transform = TRUE;
     SWFDEC_LOG ("matrix: %g %g  %g %g   %g %g",
 	content->transform.xx, content->transform.yy, 
 	content->transform.xy, content->transform.yx,
 	content->transform.x0, content->transform.y0);
     swfdec_bits_get_color_transform (bits, &content->color_transform);
+    content->has_color_transform = TRUE;
 
     content->graphic = swfdec_swf_decoder_get_character (s, character);
     if (!SWFDEC_IS_GRAPHIC (content->graphic)) {
@@ -432,8 +411,8 @@ tag_func_define_button_2 (SwfdecSwfDecoder * s)
   return SWFDEC_STATUS_OK;
 }
 
-int
-tag_func_define_button (SwfdecSwfDecoder * s)
+static int
+tag_func_define_button (SwfdecSwfDecoder * s, guint tag)
 {
   SwfdecBits *bits = &s->b;
   int id;
@@ -470,6 +449,7 @@ tag_func_define_button (SwfdecSwfDecoder * s)
     content = swfdec_content_new (depth);
 
     swfdec_bits_get_matrix (bits, &content->transform, NULL);
+    content->has_transform = TRUE;
     SWFDEC_LOG ("matrix: %g %g  %g %g   %g %g",
 	content->transform.xx, content->transform.yy, 
 	content->transform.xy, content->transform.yx,
@@ -495,19 +475,7 @@ tag_func_define_button (SwfdecSwfDecoder * s)
 }
 
 static int
-tag_func_define_font_info_1 (SwfdecSwfDecoder *s)
-{
-  return tag_func_define_font_info (s, 1);
-}
-
-static int
-tag_func_define_font_info_2 (SwfdecSwfDecoder *s)
-{
-  return tag_func_define_font_info (s, 2);
-}
-
-static int
-tag_func_file_attributes (SwfdecSwfDecoder *s)
+tag_func_file_attributes (SwfdecSwfDecoder *s, guint tag)
 {
   int has_metadata, use_network;
 
@@ -524,21 +492,134 @@ tag_func_file_attributes (SwfdecSwfDecoder *s)
   return SWFDEC_STATUS_OK;
 }
 
+static int
+tag_func_export_assets (SwfdecSwfDecoder * s, guint tag)
+{
+  SwfdecBits *bits = &s->b;
+  guint count, i;
+
+  count = swfdec_bits_get_u16 (bits);
+  SWFDEC_LOG ("exporting %u assets", count);
+  for (i = 0; i < count && swfdec_bits_left (bits); i++) {
+    guint id;
+    SwfdecCharacter *object;
+    char *name;
+    id = swfdec_bits_get_u16 (bits);
+    object = swfdec_swf_decoder_get_character (s, id);
+    name = swfdec_bits_get_string_with_version (bits, s->version);
+    if (object == NULL) {
+      SWFDEC_ERROR ("cannot export id %u as %s, id wasn't found", id, name);
+      g_free (name);
+    } else if (name == NULL) {
+      SWFDEC_ERROR ("cannot export id %u, no name was given", id);
+    } else {
+      SwfdecRootExportData *data = g_new (SwfdecRootExportData, 1);
+      data->name = name;
+      data->character = object;
+      SWFDEC_LOG ("exporting %s %u as %s", G_OBJECT_TYPE_NAME (object), id, name);
+      g_object_ref (object);
+      swfdec_swf_decoder_add_root_action (s, SWFDEC_ROOT_ACTION_EXPORT, data);
+    }
+  }
+
+  return SWFDEC_STATUS_OK;
+}
+
+static int
+tag_func_do_init_action (SwfdecSwfDecoder * s, guint tag)
+{
+  SwfdecBits *bits = &s->b;
+  guint id;
+  SwfdecSprite *sprite;
+  char *name;
+
+  id = swfdec_bits_get_u16 (bits);
+  SWFDEC_LOG ("  id = %u", id);
+  sprite = swfdec_swf_decoder_get_character (s, id);
+  if (!SWFDEC_IS_SPRITE (sprite)) {
+    SWFDEC_ERROR ("character %u is not a sprite", id);
+    return SWFDEC_STATUS_OK;
+  }
+  if (sprite->init_action != NULL) {
+    SWFDEC_ERROR ("sprite %u already has an init action", id);
+    return SWFDEC_STATUS_OK;
+  }
+  name = g_strdup_printf ("InitAction %u", id);
+  sprite->init_action = swfdec_script_new_for_context (SWFDEC_AS_CONTEXT (SWFDEC_DECODER (s)->player),
+      bits, name, s->version);
+  g_free (name);
+  if (sprite->init_action) {
+    swfdec_script_ref (sprite->init_action);
+    swfdec_swf_decoder_add_root_action (s, SWFDEC_ROOT_ACTION_INIT_SCRIPT, sprite->init_action);
+  }
+
+  return SWFDEC_STATUS_OK;
+}
+
+static int
+tag_func_enqueue (SwfdecSwfDecoder *s, guint tag)
+{
+  SwfdecBuffer *buffer;
+
+  buffer = swfdec_bits_get_buffer (&s->b, -1);
+  SWFDEC_LOG ("queueing %s tag for sprite %u", swfdec_swf_decoder_get_tag_name (tag),
+      SWFDEC_CHARACTER (s->parse_sprite)->id);
+  swfdec_sprite_add_action (s->parse_sprite, tag, buffer);
+
+  return SWFDEC_STATUS_OK;
+}
+
+static int
+tag_func_show_frame (SwfdecSwfDecoder * s, guint tag)
+{
+  SWFDEC_DEBUG("show_frame %d of id %d", s->parse_sprite->parse_frame,
+      SWFDEC_CHARACTER (s->parse_sprite)->id);
+
+  s->parse_sprite->parse_frame++;
+  if (s->parse_sprite->parse_frame < s->parse_sprite->n_frames) {
+    SwfdecSpriteFrame *old = &s->parse_sprite->frames[s->parse_sprite->parse_frame - 1];
+    SwfdecSpriteFrame *new = &s->parse_sprite->frames[s->parse_sprite->parse_frame];
+    if (old->sound_head)
+      new->sound_head = g_object_ref (old->sound_head);
+  }
+  tag_func_enqueue (s, tag);
+
+  return SWFDEC_STATUS_IMAGE;
+}
+
+static int
+tag_func_do_action (SwfdecSwfDecoder * s, guint tag)
+{
+  SwfdecScript *script;
+  char *name;
+
+  name = g_strdup_printf ("Sprite%u_Frame%u", SWFDEC_CHARACTER (s->parse_sprite)->id,
+      s->parse_sprite->parse_frame);
+  script = swfdec_script_new_for_context (SWFDEC_AS_CONTEXT (SWFDEC_DECODER (s)->player), &s->b, name, s->version);
+  g_free (name);
+  if (script) {
+    swfdec_swf_decoder_add_script (s, script);
+    swfdec_sprite_add_action (s->parse_sprite, tag, swfdec_buffer_ref (script->buffer));
+  }
+
+  return SWFDEC_STATUS_OK;
+}
+
 /* may appear inside DefineSprite */
 #define SPRITE 1
 struct tag_func_struct
 {
-  char *name;
-  int (*func) (SwfdecSwfDecoder * s);
+  const char *name;
+  SwfdecTagFunc func;
   int flag;
 };
 static struct tag_func_struct tag_funcs[] = {
   [SWFDEC_TAG_END] = {"End", tag_func_end, SPRITE},
-  [SWFDEC_TAG_SHOWFRAME] = {"ShowFrame", tag_show_frame, SPRITE},
+  [SWFDEC_TAG_SHOWFRAME] = {"ShowFrame", tag_func_show_frame, SPRITE},
   [SWFDEC_TAG_DEFINESHAPE] = {"DefineShape", tag_define_shape, 0},
   [SWFDEC_TAG_FREECHARACTER] = {"FreeCharacter", NULL, 0},
   [SWFDEC_TAG_PLACEOBJECT] = {"PlaceObject", NULL, SPRITE},
-  [SWFDEC_TAG_REMOVEOBJECT] = {"RemoveObject", swfdec_spriteseg_remove_object, SPRITE},
+  [SWFDEC_TAG_REMOVEOBJECT] = {"RemoveObject", tag_func_enqueue, SPRITE},
   [SWFDEC_TAG_DEFINEBITSJPEG] = {"DefineBitsJPEG", tag_func_define_bits_jpeg, 0},
   [SWFDEC_TAG_DEFINEBUTTON] = {"DefineButton", tag_func_define_button, 0},
   [SWFDEC_TAG_JPEGTABLES] = {"JPEGTables", swfdec_image_jpegtables, 0},
@@ -547,7 +628,7 @@ static struct tag_func_struct tag_funcs[] = {
   [SWFDEC_TAG_DEFINEFONT] = {"DefineFont", tag_func_define_font, 0},
   [SWFDEC_TAG_DEFINETEXT] = {"DefineText", tag_func_define_text, 0},
   [SWFDEC_TAG_DOACTION] = {"DoAction", tag_func_do_action, SPRITE},
-  [SWFDEC_TAG_DEFINEFONTINFO] = {"DefineFontInfo", tag_func_define_font_info_1, 0},
+  [SWFDEC_TAG_DEFINEFONTINFO] = {"DefineFontInfo", tag_func_define_font_info, 0},
   [SWFDEC_TAG_DEFINESOUND] = {"DefineSound", tag_func_define_sound, 0},
   [SWFDEC_TAG_STARTSOUND] = {"StartSound", tag_func_start_sound, SPRITE},
   [SWFDEC_TAG_DEFINEBUTTONSOUND] =
@@ -557,13 +638,13 @@ static struct tag_func_struct tag_funcs[] = {
   [SWFDEC_TAG_DEFINEBITSLOSSLESS] =
       {"DefineBitsLossless", tag_func_define_bits_lossless, 0},
   [SWFDEC_TAG_DEFINEBITSJPEG2] = {"DefineBitsJPEG2", tag_func_define_bits_jpeg_2, 0},
-  [SWFDEC_TAG_DEFINESHAPE2] = {"DefineShape2", tag_define_shape_2, 0},
+  [SWFDEC_TAG_DEFINESHAPE2] = {"DefineShape2", tag_define_shape, 0},
   [SWFDEC_TAG_DEFINEBUTTONCXFORM] = {"DefineButtonCXForm", NULL, 0},
   [SWFDEC_TAG_PROTECT] = {"Protect", tag_func_protect, 0},
-  [SWFDEC_TAG_PLACEOBJECT2] = {"PlaceObject2", swfdec_spriteseg_place_object_2, SPRITE},
-  [SWFDEC_TAG_REMOVEOBJECT2] = {"RemoveObject2", swfdec_spriteseg_remove_object_2, SPRITE},
+  [SWFDEC_TAG_PLACEOBJECT2] = {"PlaceObject2", tag_func_enqueue, SPRITE},
+  [SWFDEC_TAG_REMOVEOBJECT2] = {"RemoveObject2", tag_func_enqueue, SPRITE},
   [SWFDEC_TAG_DEFINESHAPE3] = {"DefineShape3", tag_define_shape_3, 0},
-  [SWFDEC_TAG_DEFINETEXT2] = {"DefineText2", tag_func_define_text_2, 0},
+  [SWFDEC_TAG_DEFINETEXT2] = {"DefineText2", tag_func_define_text, 0},
   [SWFDEC_TAG_DEFINEBUTTON2] = {"DefineButton2", tag_func_define_button_2, 0},
   [SWFDEC_TAG_DEFINEBITSJPEG3] = {"DefineBitsJPEG3", tag_func_define_bits_jpeg_3, 0},
   [SWFDEC_TAG_DEFINEBITSLOSSLESS2] =
@@ -587,14 +668,14 @@ static struct tag_func_struct tag_funcs[] = {
   [SWFDEC_TAG_ENABLEDEBUGGER] = {"EnableDebugger", NULL, 0},
   [SWFDEC_TAG_DOINITACTION] = {"DoInitAction", tag_func_do_init_action, SPRITE},
   [SWFDEC_TAG_DEFINEVIDEOSTREAM] = {"DefineVideoStream", tag_func_define_video, 0},
-  [SWFDEC_TAG_VIDEOFRAME] = {"VideoFrame", tag_func_video_frame, 0},
-  [SWFDEC_TAG_DEFINEFONTINFO2] = {"DefineFontInfo2", tag_func_define_font_info_2, 0},
+  [SWFDEC_TAG_VIDEOFRAME] = {"VideoFrame", tag_func_video_frame, SPRITE},
+  [SWFDEC_TAG_DEFINEFONTINFO2] = {"DefineFontInfo2", tag_func_define_font_info, 0},
   [SWFDEC_TAG_MX4] = {"MX4", NULL, 0},
   [SWFDEC_TAG_ENABLEDEBUGGER2] = {"EnableDebugger2", NULL, 0},
   [SWFDEC_TAG_SCRIPTLIMITS] = {"ScriptLimits", NULL, 0},
   [SWFDEC_TAG_SETTABINDEX] = {"SetTabIndex", NULL, 0},
   [SWFDEC_TAG_FILEATTRIBUTES] = {"FileAttributes", tag_func_file_attributes, 0},
-  [SWFDEC_TAG_PLACEOBJECT3] = {"PlaceObject3", swfdec_spriteseg_place_object_3, SPRITE},
+  [SWFDEC_TAG_PLACEOBJECT3] = {"PlaceObject3", tag_func_enqueue, SPRITE},
   [SWFDEC_TAG_IMPORTASSETS2] = {"ImportAssets2", NULL, 0},
   [SWFDEC_TAG_DEFINEFONTALIGNZONES] = {"DefineFontAlignZones", NULL, 0},
   [SWFDEC_TAG_CSMTEXTSETTINGS] = {"CSMTextSettings", NULL, 0},
@@ -621,7 +702,7 @@ swfdec_swf_decoder_get_tag_name (int tag)
   return "unknown";
 }
 
-SwfdecTagFunc *
+SwfdecTagFunc
 swfdec_swf_decoder_get_tag_func (int tag)
 { 
   if (tag >= 0 && tag < n_tag_funcs) {
