@@ -24,9 +24,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include <libswfdec/swfdec_debugger.h>
-#include <libswfdec/swfdec_js.h>
-#include <libswfdec/js/jsdbgapi.h>
-#include <libswfdec/js/jsinterp.h>
+#include <libswfdec/swfdec_as_object.h>
 #include "swfdec_player_manager.h"
 #include <libswfdec-gtk/swfdec_source.h>
 
@@ -53,10 +51,10 @@ typedef enum {
 
 static void
 swfdec_player_manager_send_message (SwfdecPlayerManager *manager,
-    SwfdecMessageType type, char *format, ...) G_GNUC_PRINTF (3, 4);
+    SwfdecMessageType type, const char *format, ...) G_GNUC_PRINTF (3, 4);
 static void
 swfdec_player_manager_send_message (SwfdecPlayerManager *manager,
-    SwfdecMessageType type, char *format, ...)
+    SwfdecMessageType type, const char *format, ...)
 {
   va_list args;
   char *msg;
@@ -334,7 +332,7 @@ swfdec_player_manager_continue (SwfdecPlayerManager *manager)
 
 /*** commands ***/
 
-const char *
+static const char *
 parse_skip (const char *input)
 {
   g_assert (input);
@@ -343,7 +341,7 @@ parse_skip (const char *input)
   return input;
 }
 
-const char *
+static const char *
 parse_string (const char *input, char **output)
 {
   const char *start = input;
@@ -360,7 +358,7 @@ parse_string (const char *input, char **output)
   return parse_skip (input);
 }
 
-const char *
+static const char *
 parse_uint (const char *input, guint *output)
 {
   char *end;
@@ -444,12 +442,6 @@ command_continue (SwfdecPlayerManager *manager, const char *arg)
     swfdec_player_manager_error (manager, "Not interrupted, cannot continue");
 }
 
-static void
-set_breakpoint (gpointer script, gpointer debugger)
-{
-  swfdec_debugger_set_breakpoint (debugger, script, 0);
-}
-
 static void
 command_next (SwfdecPlayerManager *manager, const char *arg)
 {
@@ -459,6 +451,15 @@ command_next (SwfdecPlayerManager *manager, const char *arg)
     swfdec_player_manager_next (manager);
 }
 
+static void
+set_breakpoint (gpointer scriptp, gpointer debugger)
+{
+  SwfdecDebuggerScript *script = scriptp;
+
+  if (script->n_commands > 0)
+    swfdec_debugger_set_breakpoint (debugger, script, 0);
+}
+
 static void
 command_break (SwfdecPlayerManager *manager, const char *arg)
 {
@@ -487,7 +488,7 @@ command_break (SwfdecPlayerManager *manager, const char *arg)
   }
   next = parse_string (arg, &str);
   if (next) {
-    if (strcasecmp (str, "start") == 0) {
+    if (g_ascii_strcasecmp (str, "start") == 0) {
       swfdec_debugger_foreach_script (SWFDEC_DEBUGGER (manager->player), 
 	  set_breakpoint, manager->player);
       swfdec_player_manager_output (manager, "set breakpoint at start of every script");
@@ -533,25 +534,6 @@ command_delete (SwfdecPlayerManager *manager, const char *arg)
   }
 }
 
-static void
-command_stack (SwfdecPlayerManager *manager, const char *arg)
-{
-  JSStackFrame *frame = NULL;
-  guint i, min, max;
-
-  if (!swfdec_player_manager_get_interrupted (manager)) {
-    swfdec_player_manager_error (manager, "Not interrupted");
-    return;
-  }
-  JS_FrameIterator (manager->player->jscx, &frame);
-  min = 1;
-  max = frame->sp - frame->spbase;
-  for (i = min; i <= max; i++) {
-    const char *s = swfdec_js_to_string (manager->player->jscx, frame->sp[-i]);
-    swfdec_player_manager_output (manager, "%2u: %s", i, s);
-  }
-}
-
 typedef struct {
   SwfdecPlayerManager *manager;
   const char *	       string;
@@ -582,46 +564,52 @@ command_find (SwfdecPlayerManager *manager, const char *arg)
   swfdec_debugger_foreach_script (SWFDEC_DEBUGGER (manager->player), do_find, &data);
 }
 
+static gboolean
+enumerate_foreach (SwfdecAsObject *object, const char *variable, 
+    SwfdecAsValue *value, guint flags, gpointer manager)
+{
+  char *s;
+
+  s = swfdec_as_value_to_debug (value);
+  swfdec_player_manager_output (manager, "  %s: %s", variable, s);
+  g_free (s);
+  
+  return TRUE;
+}
+
 static void
 command_enumerate (SwfdecPlayerManager *manager, const char *arg)
 {
-  jsval rval;
-  JSObject *obj;
-  JSIdArray *array;
-  const char *s, *t;
-  guint i;
+  SwfdecAsValue val;
+  char *s;
+  SwfdecAsObject *object;
 
-  if (!swfdec_js_run (manager->player, arg, &rval)) {
-    swfdec_player_manager_error (manager, "Invalid command");
+  if (arg == NULL)
     return;
-  }
-  if (!JSVAL_IS_OBJECT (rval)) {
-    swfdec_player_manager_error (manager, "Given expression is not an object");
+
+  swfdec_as_context_eval (SWFDEC_AS_CONTEXT (manager->player), NULL, arg, &val);
+  s = swfdec_as_value_to_debug (&val);
+  swfdec_player_manager_output (manager, "%s", s);
+  g_free (s);
+  if (!SWFDEC_AS_VALUE_IS_OBJECT (&val))
     return;
+  object = SWFDEC_AS_VALUE_GET_OBJECT (&val);
+  swfdec_as_object_foreach (object, enumerate_foreach, manager);
+}
+
+static void
+command_where (SwfdecPlayerManager *manager, const char *arg)
+{
+  SwfdecAsFrame *frame;
+  guint i = 0;
+
+  for (frame = swfdec_as_context_get_frame (SWFDEC_AS_CONTEXT (manager->player)); frame;
+      frame = swfdec_as_frame_get_next (frame)) {
+    char *s = swfdec_as_object_get_debug (SWFDEC_AS_OBJECT (frame));
+    i++;
+    swfdec_player_manager_output (manager, "%u: %s", i, s);
+    g_free (s);
   }
-  obj = JSVAL_TO_OBJECT (rval);
-  array = JS_Enumerate (manager->player->jscx, obj);
-  if (array == NULL) {
-    swfdec_player_manager_error (manager, "Error enumerating");
-    return;
-  }
-  s = swfdec_js_to_string (manager->player->jscx, rval);
-  if (s == NULL) {
-    swfdec_player_manager_error (manager, "Cannot convert object to string");
-  }
-  swfdec_player_manager_output (manager, "properties for %s:", s);
-  for (i = 0; i < array->length; i++) {
-    if (!JS_IdToValue (manager->player->jscx, array->vector[i], &rval))
-      continue;
-    s = swfdec_js_to_string (manager->player->jscx, rval);
-    if (s == NULL)
-      continue;
-    if (!JS_GetProperty (manager->player->jscx, obj, s, &rval) ||
-	!(t = swfdec_js_to_string (manager->player->jscx, rval)))
-      t = "<error querying value>";
-    swfdec_player_manager_output (manager, "  %s: %s", s, t);
-  }
-  JS_DestroyIdArray (manager->player->jscx, array);
 }
 
 static void command_help (SwfdecPlayerManager *manager, const char *arg);
@@ -641,9 +629,9 @@ struct {
   { "delete",	command_delete,		"delete a breakpoint" },
   { "continue",	command_continue,	"continue when stopped inside a breakpoint" },
   { "next",	command_next,		"step forward one command when stopped inside a breakpoint" },
-  { "stack",	command_stack,		"print the arguments on the stack" },
   { "find",	command_find,		"find the given argument verbatim in all scripts" },
   { "enumerate",command_enumerate,    	"enumerate all properties of the given object" },
+  { "where",	command_where,    	"print a backtrace" },
 };
 
 static void
